@@ -3,9 +3,9 @@ import pytest
 import os
 
 from collections import namedtuple
-from shutil import rmtree
+from shutil import rmtree, copyfile
 
-from .testing_utils import get_test_data
+from .testing_utils import get_test_data, get_treesapp_file
 
 
 @pytest.fixture(scope="class")
@@ -66,7 +66,7 @@ class TreeSAPPClassTester(unittest.TestCase):
 
     def test_furnish_with_arguments(self):
         args = namedtuple("args", ["molecule", "output", "input", "executables"])
-        args.input = self.fasta
+        args.input = [self.fasta]
         args.output = self.output_dir
         args.molecule = "prot"
         args.executables = {'prodigal': '/home/connor/bin/prodigal', 'BMGE.jar': '/usr/local/bin/BMGE.jar',
@@ -140,7 +140,7 @@ class EvaluatorTester(unittest.TestCase):
         self.eval.var_output_dir = self.output_dir
         # Set the instance's reference package
         self.eval.ref_pkg = ReferencePackage()
-        self.eval.ref_pkg.f__json = get_test_data(os.path.join("refpkgs", "PuhA_build.pkl"))
+        self.eval.ref_pkg.f__pkl = get_test_data(os.path.join("refpkgs", "PuhA_build.pkl"))
         self.eval.ref_pkg.slurp()
 
         return
@@ -158,10 +158,69 @@ class EvaluatorTester(unittest.TestCase):
         self.assertEqual(os.path.join(self.output_dir, rpp, taxon) + os.sep,
                          taxa_test.intermediates_dir)
         self.assertEqual(os.path.join(self.output_dir, rpp, taxon,
-                                      "treesapp_output", "final_outputs", "marker_contig_map.tsv"),
+                                      "treesapp_output", "final_outputs", "classifications.tsv"),
                          taxa_test.classification_table)
         return
 
 
+class PurityTester(unittest.TestCase):
+    def setUp(self) -> None:
+        from treesapp.classy import Purity
+        self.purity_inst = Purity()
+        copyfile(get_test_data(os.path.join("refpkgs", "PuhA_build.pkl")), "PuhA_build.pkl")
+        return
+
+    def tearDown(self) -> None:
+        if os.path.isfile("PuhA_build.pkl"):
+            os.remove("PuhA_build.pkl")
+        return
+
+    def test_check_purity_arguments(self):
+        from treesapp.treesapp_args import add_purity_arguments, TreeSAPPArgumentParser
+        # Set up the command-line arguments
+        cli_args = ["--fastx_input", get_treesapp_file("dev_utils/TIGRFAM_seed_named.faa"),
+                    "--extra_info", get_treesapp_file("dev_utils/TIGRFAM_info.tsv"),
+                    "--output", "./TreeSAPP_purity",
+                    "--trim_align", "--molecule", "prot"]
+        parser = TreeSAPPArgumentParser()
+        add_purity_arguments(parser)
+        # Test the file in the current working directory
+        args = parser.parse_args(cli_args + ["--refpkg_path", "PuhA_build.pkl"])
+        self.purity_inst.furnish_with_arguments(args)
+        self.purity_inst.check_purity_arguments(args)
+        self.assertEqual(os.getcwd(), self.purity_inst.refpkg_dir)
+        # Ensure the './' is handled properly
+        args = parser.parse_args(cli_args + ["--refpkg_path", "./PuhA_build.pkl"])
+        self.purity_inst.furnish_with_arguments(args)
+        self.purity_inst.check_purity_arguments(args)
+        self.assertEqual(os.getcwd(), self.purity_inst.refpkg_dir)
+        return
+
+
+class AbundanceTester(unittest.TestCase):
+    def test_fetch_refpkgs_used(self):
+        from treesapp.classy import Abundance
+        from .testing_utils import get_treesapp_root, get_test_data
+        mock_abund = Abundance()
+        assign_output_intermediates = get_test_data(os.path.join("test_output_TarA", "intermediates"))
+        test_refpkg_dir = get_test_data("refpkgs")
+        # Test using the packaged reference packages
+        mock_abund.fetch_refpkgs_used()
+        self.assertEqual(os.path.join(get_treesapp_root(), "data"), os.path.dirname(mock_abund.refpkg_dir))
+        self.assertEqual(33, len(mock_abund.target_refpkgs))
+
+        # Test with the reference packages stashed in the treesapp assign output
+        mock_abund.var_output_dir = assign_output_intermediates
+        mock_abund.fetch_refpkgs_used()
+        self.assertEqual(assign_output_intermediates, os.path.dirname(mock_abund.refpkg_dir))
+        self.assertEqual(["DsrAB", "McrA", "McrB"], sorted(list(mock_abund.target_refpkgs)))
+
+        # Test with the reference package path provided
+        mock_abund.fetch_refpkgs_used(test_refpkg_dir)
+        self.assertEqual(test_refpkg_dir, os.path.dirname(mock_abund.refpkg_dir))
+        self.assertEqual(5, len(mock_abund.target_refpkgs))
+        return
+
+
 if __name__ == '__main__':
     unittest.main()
