@@ -17,6 +17,7 @@ class TreesappTester(unittest.TestCase):
         self.mcra_pkl = get_test_data(os.path.join("refpkgs", "McrA_build.pkl"))
         self.mcrb_pkl = get_test_data(os.path.join("refpkgs", "McrB_build.pkl"))
         self.puha_pkl = get_test_data(os.path.join("refpkgs", "PuhA_build.pkl"))
+        self.xmoa_pkl = get_test_data(os.path.join("refpkgs", "XmoA_build.pkl"))
 
         # Output examples
         self.ts_assign_output = get_test_data("test_output_TarA/")
@@ -37,7 +38,7 @@ class TreesappTester(unittest.TestCase):
         from treesapp.file_parsers import read_classification_table
         from treesapp.phylo_seq import assignments_to_pqueries
         from .testing_utils import get_test_data
-        classification_table = os.path.join(self.ts_assign_output, "final_outputs", "marker_contig_map.tsv")
+        classification_table = os.path.join(self.ts_assign_output, "final_outputs", "classifications.tsv")
         # Copy the classification table to replace after overwrite
         copyfile(classification_table, os.path.join(self.ts_assign_output, "tmp.tsv"))
         pre_lines = read_classification_table(get_test_data(classification_table))
@@ -45,6 +46,8 @@ class TreesappTester(unittest.TestCase):
                                   "--reads", get_test_data("test_TarA.1.fq"),
                                   "--reverse", get_test_data("test_TarA.2.fq"),
                                   "--pairing", "pe",
+                                  "--report", "update",
+                                  "--metric", "fpkm",
                                   "--num_procs", str(self.num_procs),
                                   "--overwrite"]
         abund_dict = abundance(abundance_command_list)["test_TarA"]
@@ -61,6 +64,7 @@ class TreesappTester(unittest.TestCase):
                                   "--reads", get_test_data("test_TarA.fastq.gz"),
                                   "--pairing", "pe",
                                   "--metric", "tpm",
+                                  "--report", "append",
                                   "--num_procs", str(self.num_procs),
                                   "--delete", "--overwrite"]
         abundance(abundance_command_list)
@@ -69,7 +73,7 @@ class TreesappTester(unittest.TestCase):
         abundant_pqueries = []
         for rp in pqueries:
             abundant_pqueries += [pq for pq in pqueries[rp] if pq.abundance > 0.0]
-        self.assertEqual(33702, round(sum([pq.abundance for pq in abundant_pqueries])))
+        self.assertEqual(33850, round(sum([pq.abundance for pq in abundant_pqueries])))
 
         # Replace the classification table
         copyfile(os.path.join(self.ts_assign_output, "tmp.tsv"), classification_table)
@@ -77,7 +81,7 @@ class TreesappTester(unittest.TestCase):
         return
 
     def test_assign(self):
-        ref_pkgs = ["McrA", "M0702", "S0001"]
+        ref_pkgs = ["McrA", "McrB", "DsrAB"]
         from treesapp import assign
         from .testing_utils import get_test_data
         from treesapp.file_parsers import read_classification_table
@@ -86,23 +90,21 @@ class TreesappTester(unittest.TestCase):
                                 "--targets", ','.join(ref_pkgs),
                                 "--refpkg_dir", self.refpkg_dir,
                                 "--num_procs", str(self.num_procs),
-                                "-m", "prot",
                                 "--hmm_coverage", str(20),
                                 "--output", "./TreeSAPP_assign/",
                                 "--stringency", "relaxed",
                                 "--placement_summary", "max_lwr",
                                 "--trim_align", "--overwrite", "--delete", "--svm"]
         assign.assign(assign_commands_list)
-        lines = read_classification_table("./TreeSAPP_assign/final_outputs/marker_contig_map.tsv")
+        lines = read_classification_table("./TreeSAPP_assign/final_outputs/classifications.tsv")
         self.assertEqual(15, len(lines))
         self.assertTrue(os.path.isfile("./TreeSAPP_assign/final_outputs/marker_test_suite_classified.faa"))
 
         # Test nucleotide sequence input
-        ref_pkgs = ["M0701", "M0702"]
+        ref_pkgs = ["McrA", "McrB"]
         assign_commands_list = ["--fastx_input", self.nt_test_fa,
                                 "--targets", ','.join(ref_pkgs),
                                 "--num_procs", str(self.num_procs),
-                                "-m", "dna",
                                 "--output", "./TreeSAPP_assign/",
                                 "--stringency", "strict",
                                 "--trim_align", "--overwrite", "--delete"]
@@ -112,7 +114,7 @@ class TreesappTester(unittest.TestCase):
                                  "--reads", get_test_data("SRR3669912_1.fastq"),
                                  "--reverse", get_test_data("SRR3669912_2.fastq")]
         assign.assign(assign_commands_list)
-        assignments_tbl = "./TreeSAPP_assign/final_outputs/marker_contig_map.tsv"
+        assignments_tbl = "./TreeSAPP_assign/final_outputs/classifications.tsv"
         lines = read_classification_table(assignments_tbl)
         self.assertEqual(6, len(lines))
         classified_seqs = set()
@@ -125,6 +127,7 @@ class TreesappTester(unittest.TestCase):
 
     def test_colour(self):
         from treesapp.commands import colour
+        # Test creating iTOL files for a taxonomic rank
         colour_commands = ["-r", self.mcra_pkl, self.mcrb_pkl,
                            "-l", "family",
                            "-o", "./TreeSAPP_colour",
@@ -132,17 +135,26 @@ class TreesappTester(unittest.TestCase):
                            "--min_proportion", str(0.01),
                            "--no_polyphyletic"]
         colour(colour_commands)
-        self.assertEqual(True, True)
+        self.assertTrue(os.path.isfile(os.path.join("TreeSAPP_colour", "McrA_family_colour_strip.txt")))
 
-    def test_colour_phenotypes(self):
-        from treesapp.commands import colour
-        from .testing_utils import get_test_data
-        colour_commands = ["-r", self.mcra_pkl,
+        # Test a failure when the desired attribute doesn't exist in the reference package(s)
+        with pytest.raises(AssertionError):
+            colour(["-r", self.mcra_pkl, self.mcrb_pkl,
+                    "-o", "./TreeSAPP_colour",
+                    "--palette", "viridis",
+                    "--attribute", "Pathway"])
+
+        # Test creating strip and styles file for an annotated feature
+        colour_commands = ["-r", self.xmoa_pkl,
                            "-o", "./TreeSAPP_colour",
                            "--palette", "viridis",
-                           "--taxa_map", get_test_data("Mcr_taxonomy-phenotype_map.tsv")]
+                           "--attribute", "Paralog"]
         colour(colour_commands)
-        self.assertTrue(True)
+        itol_strip_file = os.path.join("TreeSAPP_colour", "XmoA_Paralog_colour_strip.txt")
+        self.assertTrue(os.path.isfile(itol_strip_file))
+        with open(itol_strip_file) as strip_dat:
+            self.assertEqual(25, len(strip_dat.readlines()))
+        return
 
     def test_create(self):
         from treesapp.commands import create
@@ -154,7 +166,6 @@ class TreesappTester(unittest.TestCase):
                                 "--refpkg_name", "Crt",
                                 "--similarity", "0.95",
                                 "--bootstraps", str(0),
-                                "--molecule", "prot",
                                 "--screen", "Bacteria", "--filter", "Archaea",
                                 "--min_taxonomic_rank", 'p',
                                 "--output", "./TreeSAPP_create",
@@ -164,7 +175,7 @@ class TreesappTester(unittest.TestCase):
                                 "--overwrite", "--delete", "--deduplicate"]
         create(create_commands_list)
         test_refpkg = ReferencePackage()
-        test_refpkg.f__json = "./TreeSAPP_create/final_outputs/Crt_build.pkl"
+        test_refpkg.f__pkl = "./TreeSAPP_create/final_outputs/Crt_build.pkl"
         test_refpkg.slurp()
         self.assertTrue(test_refpkg.validate())
         self.assertEqual(77, test_refpkg.num_seqs)
@@ -178,7 +189,6 @@ class TreesappTester(unittest.TestCase):
                                 "--similarity", "0.90",
                                 "--bootstraps", str(4),
                                 "--profile", get_test_data("PuhA_search.hmm"),
-                                "--molecule", "prot",
                                 "--screen", "Bacteria,Archaea",
                                 "--num_procs", str(self.num_procs),
                                 "--raxml_model", "LG+F+R4",
@@ -188,7 +198,7 @@ class TreesappTester(unittest.TestCase):
                                 "--overwrite", "--delete"]
         create(create_commands_list)
         test_refpkg = ReferencePackage()
-        test_refpkg.f__json = "./TreeSAPP_create/final_outputs/PuhA_build.pkl"
+        test_refpkg.f__pkl = "./TreeSAPP_create/final_outputs/PuhA_build.pkl"
         test_refpkg.slurp()
         self.assertTrue(test_refpkg.validate())
         self.assertEqual(39, test_refpkg.num_seqs)
@@ -201,7 +211,6 @@ class TreesappTester(unittest.TestCase):
                     "--refpkg_name", "PF01280",
                     "--similarity", "0.95",
                     "--bootstraps", str(0),
-                    "--molecule", "prot",
                     "--screen", "Archaea", "--filter", "Bacteria,Eukaryota",
                     "--min_taxonomic_rank", 'g',
                     "--num_procs", str(self.num_procs),
@@ -209,7 +218,7 @@ class TreesappTester(unittest.TestCase):
                     "--trim_align", "--cluster", "--fast", "--headless", "--overwrite", "--delete"]
         create(cmd_list)
         test_refpkg = ReferencePackage()
-        test_refpkg.f__json = "./TreeSAPP_create/final_outputs/PF01280_build.pkl"
+        test_refpkg.f__pkl = "./TreeSAPP_create/final_outputs/PF01280_build.pkl"
         test_refpkg.slurp()
         self.assertTrue(test_refpkg.validate())
         self.assertEqual(51, test_refpkg.num_seqs)
@@ -245,27 +254,26 @@ class TreesappTester(unittest.TestCase):
         from treesapp.commands import layer
         from .testing_utils import get_test_data
         from treesapp.file_parsers import read_classification_table
-        # Layering annotations from multiple reference packages
-        original_table = os.path.join(self.ts_assign_output, "final_outputs", "marker_contig_map.tsv")
-        layered_table = os.path.join(self.ts_assign_output, "final_outputs",
-                                     "extra_annotated_marker_contig_map.tsv")
-        pre_lines = read_classification_table(get_test_data(original_table))
+
+        # Layering annotations from multiple reference packages without feature_annotations
         layer_command_list = ["--treesapp_output", self.ts_assign_output]
-        layer_command_list += ["--refpkg_dir", self.refpkg_dir]
         layer(layer_command_list)
-        post_lines = read_classification_table(get_test_data(layered_table))
-        self.assertEqual(len(pre_lines), len(post_lines))
 
         # With a different reference package, XmoA, just to be sure
         classification_table = os.path.join(get_test_data("p_amoA_FunGene9.5_isolates_assign/"),
-                                            "final_outputs", "marker_contig_map.tsv")
+                                            "final_outputs", "classifications.tsv")
+        layered_table = os.path.join(get_test_data("p_amoA_FunGene9.5_isolates_assign/"), "final_outputs",
+                                     "layered_classifications.tsv")
         pre_lines = read_classification_table(get_test_data(classification_table))
-        layer_command_list = ["--colours_style", get_test_data("XmoA_Function.txt"),
-                              "--treesapp_output", get_test_data("p_amoA_FunGene9.5_isolates_assign/"),
+        layer_command_list = ["--treesapp_output", get_test_data("p_amoA_FunGene9.5_isolates_assign/"),
                               "--refpkg_dir", self.refpkg_dir]
         layer(layer_command_list)
         post_lines = read_classification_table(get_test_data(classification_table))
+
+        layered_classifications = read_classification_table(layered_table)
         self.assertEqual(len(pre_lines), len(post_lines))
+        for line in layered_classifications:
+            self.assertEqual(13, len(line))
         return
 
     def test_purity(self):
@@ -284,21 +292,54 @@ class TreesappTester(unittest.TestCase):
     def test_package(self):
         from treesapp.commands import package
         from treesapp.refpkg import ReferencePackage
-        view_command_list = ["view", "lineage_ids",
+        from treesapp.clade_annotation import CladeAnnotation
+        from .testing_utils import get_test_data
+
+        test_refpkg = ReferencePackage()
+        output_dir = "./TreeSAPP_package" + os.sep
+        test_refpkg.f__pkl = output_dir + "McrA_build.pkl"
+        edit_command_list = ["edit",
+                             "refpkg_code", "M0701",
                              "--refpkg_path", self.mcra_pkl,
-                             "--output", "./TreeSAPP_package"]
-        package(view_command_list)
+                             "--output", output_dir]
+        package(edit_command_list)
+
+        # Test editing a component file's contents
         edit_command_list = ["edit",
                              "f__msa", self.aa_test_fa,
                              "--refpkg_path", self.mcra_pkl,
-                             "--output", "./TreeSAPP_package"]
-        package(edit_command_list)
-        test_refpkg = ReferencePackage()
-        test_refpkg.f__json = "./TreeSAPP_package/McrA_build.pkl"
+                             "--output", output_dir]
+        with pytest.raises(SystemExit):
+            package(edit_command_list)
         test_refpkg.slurp()
-        self.assertFalse(test_refpkg.validate())
+        self.assertTrue(test_refpkg.validate())
         with pytest.raises(SystemExit):
             package(["rename", "-h"])
+
+        # Test adding to a reference package's feature_annotations attribute
+        edit_command_list = ["edit",
+                             "feature_annotations", "Function",
+                             "--taxa_map", get_test_data("Mcr_taxonomy-phenotype_map.tsv"),
+                             "--refpkg_path", self.mcra_pkl,
+                             "--output", output_dir,
+                             "--overwrite"]
+        package(edit_command_list)
+        test_refpkg.slurp()
+        self.assertIsInstance(test_refpkg.feature_annotations, dict)
+        self.assertTrue('Function' in test_refpkg.feature_annotations)
+        self.assertEqual(5, len(test_refpkg.feature_annotations["Function"]))
+        self.assertIsInstance(test_refpkg.feature_annotations["Function"][0], CladeAnnotation)
+        for ca in test_refpkg.feature_annotations["Function"]:
+            if ca.name == "Short-chain alkane oxidation":
+                self.assertEqual(2, len(ca.members))
+
+        # Test view with a complex attribute (feature_annotations)
+        view_command_list = ["view",
+                             "lineage_ids", "feature_annotations", "tree",
+                             "--refpkg_path", output_dir + os.path.basename(self.mcra_pkl),
+                             "--output", output_dir]
+        package(view_command_list)
+
         return
 
     def test_train(self):
@@ -315,7 +356,6 @@ class TreesappTester(unittest.TestCase):
                               "--profile",
                               "--max_examples", str(max_ex),
                               "--num_proc", str(self.num_procs),
-                              "--molecule", "prot",
                               "--svm_kernel", "rbf",
                               "--classifier", "bin",
                               "--trim_align", "--delete", "--overwrite"]
@@ -341,15 +381,15 @@ class TreesappTester(unittest.TestCase):
                                "--treesapp_output", get_test_data("assign_SwissProt_PuhA/"),
                                "--output", "./TreeSAPP_update",
                                "--num_proc", str(self.num_procs),
-                               "--molecule", "prot",
                                "--trim_align", "--cluster", "--fast", "--headless",
                                "--overwrite", "--delete", "--skip_assign", "--resolve"]
         update(update_command_list)
         test_refpkg = ReferencePackage()
-        test_refpkg.f__json = "./TreeSAPP_update/final_outputs/PuhA_build.pkl"
+        test_refpkg.f__pkl = "./TreeSAPP_update/final_outputs/PuhA_build.pkl"
         test_refpkg.slurp()
         self.assertTrue(test_refpkg.validate())
         self.assertEqual(49, test_refpkg.num_seqs)
+        self.assertEqual(2, len(test_refpkg.feature_annotations["test"]))
 
         # Test the workflow when seqs2lineage table is provided and no training
         update_command_list = ["--refpkg_path", self.puha_pkl,
@@ -357,13 +397,12 @@ class TreesappTester(unittest.TestCase):
                                "--seqs2lineage", get_test_data("SwissProt_PuhA_seqs2lineage.txt"),
                                "--output", "./TreeSAPP_update",
                                "--num_procs", str(self.num_procs),
-                               "--molecule", "prot",
                                "-b", str(0),
                                "--trim_align", "--cluster", "--fast", "--headless",
                                "--overwrite", "--delete", "--skip_assign"]
         update(update_command_list)
         test_refpkg = ReferencePackage()
-        test_refpkg.f__json = "./TreeSAPP_update/final_outputs/PuhA_build.pkl"
+        test_refpkg.f__pkl = "./TreeSAPP_update/final_outputs/PuhA_build.pkl"
         test_refpkg.slurp()
         self.assertTrue(test_refpkg.validate())
         self.assertEqual(49, test_refpkg.num_seqs)
