@@ -1,11 +1,13 @@
 import os
+import pytest
 import unittest
 
+from .testing_utils import get_test_data
+
 
 class FastaTester(unittest.TestCase):
     def setUp(self) -> None:
         from treesapp.fasta import load_fasta_header_regexes
-        from .testing_utils import get_test_data
         self.header_regexes = load_fasta_header_regexes(code_name="PuhA")
 
         self.test_fa = get_test_data("McrA_eval.faa")
@@ -90,17 +92,6 @@ class FastaTester(unittest.TestCase):
                          seq_info_tuple.description)
         return
 
-    def test_read_fasta_to_dict(self):
-        from treesapp.fasta import read_fasta_to_dict
-
-        # Test normal functionality with a fasta file
-        fa_dict = read_fasta_to_dict(self.test_fa)
-        self.assertEqual(236, len(fa_dict))
-
-        # Test reading a file that doesn't exist
-        self.assertEqual({}, read_fasta_to_dict("./fake_file.fa"))
-        return
-
     def test_split_fa(self):
         from treesapp.fasta import split_fa, read_fasta_to_dict
         # Test splitting by number of files
@@ -202,6 +193,12 @@ class HeaderTester(unittest.TestCase):
         h.original = self.eggnog_three
         h.find_accession()
         self.assertEqual(self.eggnog_three, h.version)
+
+        bin_ex = "SI072_165m_bin.14_k147_157247_1"
+        h = Header(bin_ex)
+        h.find_accession()
+        self.assertEqual("SI072_165m_bin", h.accession)
+        self.assertEqual(bin_ex, h.version)
         return
 
     def test_get_info(self):
@@ -212,5 +209,78 @@ class HeaderTester(unittest.TestCase):
         return
 
 
+class FastaUtilitiesTester(unittest.TestCase):
+    def setUp(self) -> None:
+        self.test_aa_fa = get_test_data("McrA_eval.faa")
+        self.test_nuc_fa = get_test_data("marker_test_suite.fna")
+        self.troublesome_fa = get_test_data("fasta_parser_test.fasta")
+        return
+
+    def test_read_fasta_to_dict(self):
+        from treesapp.fasta import read_fasta_to_dict
+
+        # Test normal functionality with a fasta file
+        fa_dict = read_fasta_to_dict(self.test_aa_fa)
+        self.assertEqual(236, len(fa_dict))
+
+        fa_dict = read_fasta_to_dict(self.test_nuc_fa, num_records=4)
+        self.assertEqual(4, len(fa_dict))
+
+        # Test reading a file that doesn't exist
+        self.assertEqual({}, read_fasta_to_dict("./fake_file.fa"))
+        return
+
+    def test_read_fastq_to_dict(self):
+        from treesapp.fasta import read_fastq_to_dict
+        # Test a FASTQ file
+        fasta_dict = read_fastq_to_dict(fastq_file=get_test_data("SRR3669912_1.fastq"),
+                                        num_records=80)
+        self.assertEqual(80, len(fasta_dict))
+        return
+
+    def test_read_fastx_to_dict(self):
+        from treesapp.fasta import read_fastx_to_dict
+        # Test file doesn't exist
+        self.assertEqual({}, read_fastx_to_dict("test_data" + os.sep + "fake.fa"))
+        # Test a file that is neither a FASTA or FASTQ
+        with pytest.raises(SystemExit):
+            read_fastx_to_dict(fastx=get_test_data("test.sto"))
+        # Test a FASTQ file
+        fasta_dict = read_fastx_to_dict(fastx=get_test_data("SRR3669912_1.fastq"))
+        self.assertEqual(160, len(fasta_dict))
+        # Test a FASTA file
+        fasta_dict = read_fastx_to_dict(self.test_aa_fa)
+        self.assertEqual(236, len(fasta_dict))
+        return
+
+    def test_guess_sequence_type(self):
+        from treesapp.fasta import guess_sequence_type, read_fasta_to_dict
+        # Test bad keyword argument
+        with pytest.raises(ValueError):
+            guess_sequence_type(fasta=self.test_aa_fa)
+
+        with pytest.raises(SystemExit):
+            guess_sequence_type(fastx_file="tests/not_testing.fasta")
+
+        # Test with fasta file for amino acid sequences
+        seq_type = guess_sequence_type(fastx_file=self.test_aa_fa)
+        self.assertEqual("prot", seq_type)
+
+        # Test with fasta file and nucleotide sequences
+        seq_type = guess_sequence_type(fastx_file=self.test_nuc_fa)
+        self.assertEqual("dna", seq_type)
+
+        # Test with dictionary and a lot of ambiguity characters
+        fa_dict = read_fasta_to_dict(self.troublesome_fa)
+        seq_type = guess_sequence_type(fasta_dict=fa_dict)
+        self.assertEqual('', seq_type)
+
+        # Test with multiple sequence types
+        seq_type = guess_sequence_type(fasta_dict={"s1": "AAAACGGGATCGAGCTACG",
+                                                   "s2": "GORVVVLMMMNOPLC"})
+        self.assertEqual('', seq_type)
+        return
+
+
 if __name__ == '__main__':
     unittest.main()
