@@ -2,26 +2,10 @@ import os
 import unittest
 import pytest
 
-from .testing_utils import get_test_data, get_treesapp_root
+from .testing_utils import get_test_data
 
 
 class TreesappTester(unittest.TestCase):
-    def test_gather_ref_packages(self):
-        from treesapp.file_parsers import gather_ref_packages
-        from . import testing_utils as utils
-        refpkg_dir = utils.get_test_data(os.path.join("refpkgs"))
-        #
-        refpkg_dict = gather_ref_packages(refpkg_data_dir=refpkg_dir, targets=["McrA"])
-        self.assertEqual(1, len(refpkg_dict))
-
-        # Test to ensure all core reference packages are present in the main refpkg pkl directory
-        core = {"McrA", "McrB", "McrG", "DsrAB", "XmoA",
-                "NapA", "NxrA", "NifD", "NirK", "NirS", "NxrB", "NorB", "NosZ",
-                "HydA", "PilA"}
-        refpkg_dict = gather_ref_packages(refpkg_data_dir=os.path.join(get_treesapp_root(), "data"))
-        self.assertEqual(0, len(core.difference(set(refpkg_dict.keys()))))
-        return
-
     def test_read_linclust_clusters(self):
         from treesapp.file_parsers import read_linclust_clusters
         with pytest.raises(SystemExit):
@@ -51,6 +35,9 @@ class TreesappTester(unittest.TestCase):
 
     def test_read_graftm_classifications(self):
         from treesapp.file_parsers import read_graftm_classifications
+        assignments = read_graftm_classifications(get_test_data("graftm_raw_read_tax.tsv"))
+        self.assertEqual(8, len(assignments))
+        self.assertEqual(9, len(sum(assignments.values(), [])))
         return
 
     def test_grab_graftm_taxa(self):
@@ -98,7 +85,10 @@ class TreesappTester(unittest.TestCase):
 
     def test_load_classifified_sequences_from_assign_output(self):
         from treesapp.file_parsers import load_classified_sequences_from_assign_output
-        refpkg_pquery_map = load_classified_sequences_from_assign_output(get_test_data("marker_test_results"))
+        from treesapp.classy import TreeSAPP
+        assigner_instance = TreeSAPP("phylotu")
+        refpkg_pquery_map = load_classified_sequences_from_assign_output(get_test_data("marker_test_results"),
+                                                                         assigner_instance)
         self.assertEqual(["McrA", "McrB"], list(refpkg_pquery_map.keys()))
         self.assertEqual(13, sum([len(pqueries) for pqueries in refpkg_pquery_map.values()]))
         for rp, pqueries in refpkg_pquery_map.items():
@@ -108,6 +98,7 @@ class TreesappTester(unittest.TestCase):
 
         # Test refpkg filtering
         refpkg_pquery_map = load_classified_sequences_from_assign_output(get_test_data("test_output_TarA"),
+                                                                         assigner_instance,
                                                                          refpkg_name="DsrAB")
         self.assertEqual(70, len(refpkg_pquery_map["DsrAB"]))
         for rp, pqueries in refpkg_pquery_map.items():
@@ -118,7 +109,10 @@ class TreesappTester(unittest.TestCase):
 
     def test_write_classification_table(self):
         from treesapp.file_parsers import write_classification_table, load_classified_sequences_from_assign_output
-        refpkg_pquery_map = load_classified_sequences_from_assign_output(get_test_data("marker_test_results"))
+        from treesapp.classy import TreeSAPP
+        assigner_instance = TreeSAPP("phylotu")
+        refpkg_pquery_map = load_classified_sequences_from_assign_output(get_test_data("marker_test_results"),
+                                                                         assigner_instance)
         test_output = "tests/tmp_classifications.tsv"
         write_classification_table(tree_saps=refpkg_pquery_map, output_file=test_output, sample_name="Testing")
         with open(test_output) as t_out:
@@ -137,6 +131,33 @@ class TreesappTester(unittest.TestCase):
             os.remove(test_output)
         return
 
+    def test_read_phenotypes_map(self):
+        from treesapp.file_parsers import read_phenotypes
+        mcra_metabolism = get_test_data("Mcr_taxonomy-phenotype_map.tsv")
+
+        phenotypes = read_phenotypes(mcra_metabolism)
+        self.assertIsInstance(cls=dict, obj=phenotypes)
+        self.assertEqual(19, len(phenotypes))
+
+    def test_read_lineage_map(self):
+        from treesapp.file_parsers import read_lineage_map
+        mock_lineage_map = get_test_data("McrA_lineage_ids - GTDB_map.tsv")
+        lineages = read_lineage_map(mock_lineage_map)
+        self.assertEqual(149, len(lineages))
+        return
+
+    def test_read_lineage_ids(self):
+        from treesapp.file_parsers import read_lineage_ids
+        mock_lineages = get_test_data("McrA_lineage_ids.tsv")
+        lineages = read_lineage_ids(mock_lineages)
+        self.assertIsInstance(lineages, dict)
+        self.assertEqual(50, len(lineages))
+        self.assertEqual([str(x) for x in range(1, 51)], list(lineages.keys()))
+
+        with pytest.raises(SystemExit):
+            read_lineage_ids(get_test_data("McrA_lineage_ids - GTDB_map.tsv"))
+        return
+
 
 if __name__ == '__main__':
     unittest.main()
