@@ -1,5 +1,3 @@
-#!/usr/bin/env python3
-
 __author__ = "Connor Morgan-Lang"
 __maintainer__ = "Connor Morgan-Lang"
 __license__ = "GPL-3.0"
@@ -14,13 +12,13 @@ import logging
 from collections import namedtuple
 
 import pyfastx
+import pandas as pd
 from numpy import array as np_array
-from sklearn import preprocessing
 
 from treesapp import abundance
 from treesapp import classy
 from treesapp import phylo_seq
-from treesapp.refpkg import ReferencePackage
+from treesapp import refpkg
 from treesapp import treesapp_args
 from treesapp.entish import index_tree_edges, map_internal_nodes_leaves
 from treesapp import lca_calculations as ts_lca
@@ -30,6 +28,7 @@ from treesapp import phylo_dist
 from treesapp import utilities
 from treesapp import wrapper
 from treesapp import fasta
+from treesapp import training_utils
 from treesapp.hmmer_tbl_parser import HmmMatch
 
 
@@ -66,6 +65,11 @@ class Assigner(classy.TreeSAPP):
         :param args: object with parameters returned by argparse.parse_args()
         :return: 'args', a summary of TreeSAPP settings.
         """
+        if not os.path.isfile(self.input_sequences):
+            logging.error("FASTX input file '{}' doesn't exist.\n".format(self.input_sequences))
+            sys.exit(5)
+
+        self.find_sequence_molecule_type()
         self.classification_table = self.final_output_dir + os.sep + self.classification_tbl_name
         self.itol_out = self.output_dir + 'iTOL_output' + os.sep
         self.classified_aa_seqs = self.final_output_dir + self.sample_prefix + "_classified.faa"
@@ -92,6 +96,53 @@ class Assigner(classy.TreeSAPP):
 
         return args
 
+    def load_refpkg_classifiers(self, refpkg_dict: dict, kernel: str, threads: int, combine=False) -> None:
+        if not self.svc_filter:
+            return
+
+        untrained_refpkgs = []
+        classifiers = dict()
+        if combine:
+            training_frames = []
+            refpkg_names = set()
+            for name, ref_pkg in refpkg_dict.items():  # type: refpkg.ReferencePackage
+                refpkg_names.add(name)
+                if len(ref_pkg.training_df) == 0:
+                    continue
+                training_frames.append(ref_pkg.training_df)
+            if len(training_frames) == 0:
+                logging.error("All reference package training data frames are empty.\n"
+                              "Unable to train a classifier from combined training data.\n")
+                sys.exit(5)
+
+            classifiers = training_utils.train_classifier_from_dataframe(training_df=pd.concat(training_frames),
+                                                                         kernel=kernel,
+                                                                         num_threads=threads,
+                                                                         refpkg_names=refpkg_names)
+        else:
+            for _name, ref_pkg in refpkg_dict.items():  # type: refpkg.ReferencePackage
+                if len(ref_pkg.training_df) == 0:
+                    continue
+                classifiers.update(training_utils.train_classifier_from_dataframe(training_df=ref_pkg.training_df,
+                                                                                  kernel=kernel,
+                                                                                  num_threads=threads))
+
+        # Set the svc attribute of each ReferencePackage to its respective classifier
+        for _name, ref_pkg in refpkg_dict.items():
+            try:
+                ref_pkg.svc = classifiers[ref_pkg.prefix]
+            except KeyError:
+                if not ref_pkg.svc:
+                    untrained_refpkgs.append(ref_pkg.prefix)
+
+        if untrained_refpkgs:
+            logging.warning("Unable to train classifiers for {} reference packages.\n".format(len(untrained_refpkgs)))
+            logging.debug("Reference packages that will not use SVC for filtering placements:\n\t"
+                          "{}\n".format("\n\t".join(untrained_refpkgs)))
+
+        return
+
+
     @staticmethod
     def define_hmm_domtbl_thresholds(args):
         thresholds_nt = namedtuple("thresholds", ["perc_aligned", "min_acc", "max_e", "max_ie", "min_score"])
@@ -378,7 +429,7 @@ def bin_hmm_matches(hmm_matches: dict, fasta_dict: dict) -> (dict, dict):
     Used for extracting query sequences that mapped to reference package HMM profiles. These are binned into groups
     based on the location on the HMM profile they mapped to such that MSAs downstream will have more conserved positions
 
-    The first nested dictionary returned "extracted_seq_dict" contains marker (i.e. refpkg names) strings mapped
+    The first nested dictionary returned "extracted_seq_dict" contains marker (i.e. ref_pkg names) strings mapped
     to bin numbers mapped to query sequence negative integer code names mapped to their extracted, or sliced, sequence.
 
     The second dictionary returned "numeric_contig_index" is used for mapping query sequence negative integer code names
@@ -457,7 +508,7 @@ def write_grouped_fastas(extracted_seq_dict: dict, numeric_contig_index: dict, r
     logging.info("Writing the grouped sequences to FASTA files... ")
 
     for marker in extracted_seq_dict:
-        refpkg = refpkg_dict[marker]  # type: ReferencePackage
+        ref_pkg = refpkg_dict[marker]  # type: refpkg.ReferencePackage
         f_acc = 0  # For counting the number of files for a marker. Will exceed groups if len(queries) > len(references)
         for group in sorted(extracted_seq_dict[marker]):
             if extracted_seq_dict[marker][group]:
@@ -469,7 +520,7 @@ def write_grouped_fastas(extracted_seq_dict: dict, numeric_contig_index: dict, r
                     # Add the query sequence to this bin's FASTA file
                     bin_fasta[str(num)] = group_sequences[num]
                     # Ensuring the number of query sequences doesn't exceed the number of reference sequences
-                    if len(bin_fasta) >= refpkg.num_seqs:
+                    if len(bin_fasta) >= ref_pkg.num_seqs:
                         fasta.write_new_fasta(bin_fasta, output_dir + marker + "_hmm_purified_group" + str(f_acc) + ".faa")
                         hmmalign_input_fastas.append(output_dir + marker + "_hmm_purified_group" + str(f_acc) + ".faa")
                         f_acc += 1
@@ -541,8 +592,10 @@ def prep_reference_packages_for_assign(refpkg_dict: dict, output_dir: str) -> No
     :return: None
     """
     for refpkg_name in refpkg_dict:
-        ref_pkg = refpkg_dict[refpkg_name]  # type: ReferencePackage
-        ref_pkg.disband(os.path.join(output_dir, ref_pkg.prefix + "_RefPkg"))
+        ref_pkg = refpkg_dict[refpkg_name]  # type: refpkg.ReferencePackage
+        prefix_dir = os.path.join(output_dir, ref_pkg.prefix + "_RefPkg")
+        ref_pkg.disband(prefix_dir)
+        ref_pkg.pickle_package(new_output_dir=prefix_dir)
     return
 
 
@@ -555,7 +608,7 @@ def get_alignment_dims(refpkg_dict: dict):
     """
     alignment_dimensions_dict = dict()
     for refpkg_name in refpkg_dict:  # type: str
-        ref_pkg = refpkg_dict[refpkg_name]  # type: ReferencePackage
+        ref_pkg = refpkg_dict[refpkg_name]  # type: refpkg.ReferencePackage
         alignment_dimensions_dict[ref_pkg.prefix] = ref_pkg.alignment_dims()
 
     return alignment_dimensions_dict
@@ -590,21 +643,21 @@ def create_ref_phy_files(refpkgs: dict, output_dir: str, query_fasta_files: list
     :param refpkgs: A dictionary of ReferencePackage instances indexed by their prefix values
     :param output_dir: Path to a directory for writing the Phylip files
     :param query_fasta_files: A list containing paths to FASTA files that are to be converted to Phylip format
-    :param ref_aln_dimensions: A dictionary of refpkg.prefix keys mapping to a tuple of the nrow, ncol for a MSA
+    :param ref_aln_dimensions: A dictionary of ref_pkg.prefix keys mapping to a tuple of the nrow, ncol for a MSA
     :return: None
     """
 
     # Convert the reference sequence alignments to .phy files for every marker identified
     for query_fasta in query_fasta_files:
         marker = re.match("(.*)_hmm_purified.*", os.path.basename(query_fasta)).group(1)
-        refpkg = refpkgs[marker]  # type: ReferencePackage
+        ref_pkg = refpkgs[marker]  # type: refpkg.ReferencePackage
 
         ref_alignment_phy = output_dir + marker + ".phy"
         if os.path.isfile(ref_alignment_phy):
             continue
 
-        num_ref_seqs, ref_align_len = ref_aln_dimensions[refpkg.prefix]
-        aligned_fasta_dict = fasta.read_fasta_to_dict(refpkg.f__msa)
+        num_ref_seqs, ref_align_len = ref_aln_dimensions[ref_pkg.prefix]
+        aligned_fasta_dict = fasta.read_fasta_to_dict(ref_pkg.f__msa)
         phy_dict = utilities.reformat_fasta_to_phy(aligned_fasta_dict)
 
         utilities.write_phy_file(ref_alignment_phy, phy_dict, (num_ref_seqs, ref_align_len))
@@ -643,20 +696,20 @@ def prepare_and_run_hmmalign(execs: dict, single_query_fasta_files: list, refpkg
         query_mfa_out = os.path.join(output_dir,
                                      re.sub('.' + re.escape(extension) + r"$", ".sto", os.path.basename(query_fa_in)))
 
-        refpkg = refpkg_dict[refpkg_name]  # type: ReferencePackage
-        if refpkg.prefix not in hmmalign_singlehit_files:
-            hmmalign_singlehit_files[refpkg.prefix] = []
+        ref_pkg = refpkg_dict[refpkg_name]  # type: refpkg.ReferencePackage
+        if ref_pkg.prefix not in hmmalign_singlehit_files:
+            hmmalign_singlehit_files[ref_pkg.prefix] = []
         try:
-            mfa_out_dict[refpkg.prefix].append(query_mfa_out)
+            mfa_out_dict[ref_pkg.prefix].append(query_mfa_out)
         except KeyError:
-            mfa_out_dict[refpkg.prefix] = [query_mfa_out]
+            mfa_out_dict[ref_pkg.prefix] = [query_mfa_out]
 
         # Get the paths to either the HMM or CM profile files
-        if refpkg.kind == "phylogenetic_rRNA":
-            task_list.append(wrapper.hmmalign_command(execs["cmalign"], refpkg.f__msa, refpkg.f__profile,
+        if ref_pkg.kind == "phylogenetic_rRNA":
+            task_list.append(wrapper.hmmalign_command(execs["cmalign"], ref_pkg.f__msa, ref_pkg.f__profile,
                                                       query_fa_in, query_mfa_out))
         else:
-            task_list.append(wrapper.hmmalign_command(execs["hmmalign"], refpkg.f__msa, refpkg.f__profile,
+            task_list.append(wrapper.hmmalign_command(execs["hmmalign"], ref_pkg.f__msa, ref_pkg.f__profile,
                                                       query_fa_in, query_mfa_out))
 
     if len(task_list) > 0:
@@ -692,7 +745,7 @@ def gather_split_msa(refpkg_names: list, align_dir: str) -> dict:
     :param refpkg_names: A list of reference package prefix attributes to look for in the align_dir
     :param align_dir: Dictionary path containing the
     :return: A dictionary containing paired reference and query multiple alignments:
-        d = {refpkg.prefix: MSA(ref.mfa, query.mfa)}
+        d = {ref_pkg.prefix: MSA(ref.mfa, query.mfa)}
     """
     split_msa_map = {}
     MSAs = namedtuple("MSAs", "ref query")
@@ -717,7 +770,7 @@ def check_for_removed_sequences(trimmed_msa_files: dict, msa_files: dict, refpkg
 
     :param trimmed_msa_files:
     :param msa_files: A dictionary containing the untrimmed MSA files indexed by reference package code (denominator)
-    :param refpkg_dict: A dictionary of ReferencePackage objects indexed by their refpkg names
+    :param refpkg_dict: A dictionary of ReferencePackage objects indexed by their ref_pkg names
     :param min_len: The minimum allowable sequence length after trimming (not including gap characters)
     :return: A dictionary of denominators, with multiple alignment dictionaries as values. Example:
         {M0702: { "McrB_hmm_purified.phy-BMGE.fasta": {'1': seq1, '2': seq2}}}
@@ -730,32 +783,32 @@ def check_for_removed_sequences(trimmed_msa_files: dict, msa_files: dict, refpkg
     logging.debug("Validating trimmed multiple sequence alignment files... ")
 
     for refpkg_name in sorted(trimmed_msa_files.keys()):
-        refpkg = refpkg_dict[refpkg_name]  # type: ReferencePackage
-        trimmed_away_seqs[refpkg.prefix] = 0
+        ref_pkg = refpkg_dict[refpkg_name]  # type: refpkg.ReferencePackage
+        trimmed_away_seqs[ref_pkg.prefix] = 0
         # Create a set of the reference sequence names
-        ref_headers = fasta.get_headers(refpkg.f__msa)
-        unique_refs = set([re.sub('_' + re.escape(refpkg.prefix), '', x)[1:] for x in ref_headers])
-        msa_passed, msa_failed, summary_str = file_parsers.validate_alignment_trimming(trimmed_msa_files[refpkg.prefix],
+        ref_headers = fasta.get_headers(ref_pkg.f__msa)
+        unique_refs = set([re.sub('_' + re.escape(ref_pkg.prefix), '', x)[1:] for x in ref_headers])
+        msa_passed, msa_failed, summary_str = file_parsers.validate_alignment_trimming(trimmed_msa_files[ref_pkg.prefix],
                                                                                        unique_refs, True, min_len)
 
         # Report the number of sequences that are removed by BMGE
-        for trimmed_msa_file in trimmed_msa_files[refpkg.prefix]:
+        for trimmed_msa_file in trimmed_msa_files[ref_pkg.prefix]:
             try:
-                prefix, tool = re.search('(' + re.escape(refpkg.prefix) + r"_.*_group\d+)-(BMGE|trimAl).fasta$",
-                                         os.path.basename(trimmed_msa_file)).groups()
+                prefix = re.search('(' + re.escape(ref_pkg.prefix) + r"_.*_group\d+)-(BMGE|trimAl).fasta$",
+                                   os.path.basename(trimmed_msa_file)).group(1)
             except TypeError:
                 logging.error("Unexpected file name format for a trimmed MSA.\n")
                 sys.exit(3)
             # Find the untrimmed query sequence MSA file - the trimmed MSA file's 'pair'
             pair = ""
-            for msa_file in msa_files[refpkg.prefix]:
+            for msa_file in msa_files[ref_pkg.prefix]:
                 if re.search(re.escape(prefix) + r'\.', msa_file):
                     pair = msa_file
                     break
             if pair:
                 if trimmed_msa_file in msa_failed:
                     untrimmed_msa_failed.append(pair)
-                trimmed_away_seqs[refpkg.prefix] += len(set(fasta.get_headers(pair)).difference(set(fasta.get_headers(trimmed_msa_file))))
+                trimmed_away_seqs[ref_pkg.prefix] += len(set(fasta.get_headers(pair)).difference(set(fasta.get_headers(trimmed_msa_file))))
             else:
                 logging.error("Unable to map trimmed MSA file '" + trimmed_msa_file + "' to its original MSA.\n")
                 sys.exit(5)
@@ -764,13 +817,13 @@ def check_for_removed_sequences(trimmed_msa_files: dict, msa_files: dict, refpkg
             if len(untrimmed_msa_failed) != len(msa_failed):
                 logging.error("Not all of the failed ({}/{}),"
                               " trimmed MSA files were mapped to their original MSAs."
-                              "\n".format(len(msa_failed), len(trimmed_msa_files[refpkg.prefix])))
+                              "\n".format(len(msa_failed), len(trimmed_msa_files[ref_pkg.prefix])))
                 sys.exit(3)
             untrimmed_msa_passed, _, _ = file_parsers.validate_alignment_trimming(untrimmed_msa_failed, unique_refs,
                                                                                   True, min_len)
             msa_passed.update(untrimmed_msa_passed)
         num_successful_alignments += len(msa_passed)
-        qc_ma_dict[refpkg.prefix] = msa_passed
+        qc_ma_dict[ref_pkg.prefix] = msa_passed
         discarded_seqs_string += summary_str
         untrimmed_msa_failed.clear()
 
@@ -859,42 +912,6 @@ def delete_files(clean_up: bool, root_dir: str, section: int) -> None:
     return
 
 
-def generate_simplebar(target_marker, tree_protein_list, itol_bar_file):
-    """
-    From the basic abundance output csv file, generate an iTOL-compatible simple bar-graph file for each leaf
-
-    :param target_marker:
-    :param tree_protein_list: A list of PQuery objects, for single sequences
-    :param itol_bar_file: The name of the file to write the simple-bar data for iTOL
-    :return:
-    """
-    leaf_abundance_sums = dict()
-
-    for tree_sap in tree_protein_list:  # type: phylo_seq.PQuery
-        if tree_sap.ref_name == target_marker and tree_sap.classified:
-            leaf_abundance_sums = tree_sap.sum_abundances_per_node(leaf_abundance_sums)
-
-    # Only make the file if there is something to write
-    if len(leaf_abundance_sums.keys()) > 0:
-        try:
-            itol_abundance_out = open(itol_bar_file, 'w')
-        except IOError:
-            logging.error("Unable to open " + itol_bar_file + " for writing.\n")
-            sys.exit(3)
-
-        # Write the header
-        header = "DATASET_SIMPLEBAR\nSEPARATOR COMMA\nDATASET_LABEL,ABUNDANCE\nCOLOR,#ff0000\n"
-        itol_abundance_out.write(header)
-        # Write the abundance sums for each leaf
-        itol_abundance_out.write("DATA\n")
-        data_lines = [','.join([str(k), str(v)]) for k, v in leaf_abundance_sums.items()]
-        itol_abundance_out.write("\n".join(data_lines))
-
-        itol_abundance_out.close()
-
-    return tree_protein_list
-
-
 def filter_placements(tree_saps: dict, refpkg_dict: dict, svc: bool, min_lwr: float, max_pendant: float) -> None:
     """
     Determines the total distance of each placement from its branch point on the tree
@@ -908,61 +925,49 @@ def filter_placements(tree_saps: dict, refpkg_dict: dict, svc: bool, min_lwr: fl
     PQueries with pendant length > max_pendant will be unclassified.
     :return: None
     """
+    # The following list must match that of training_utils.vectorize_placement_data_by_rank()
+    features = ["evalue", "hmm_cov", "leaves", "lwr", "distal", "pendant", "avg_tip_dist"]
 
     logging.info("Filtering low-quality placements... ")
     unclassified_seqs = dict()  # A dictionary tracking the seqs unclassified for each marker
 
     for refpkg_name, pqueries in tree_saps.items():  # type: (str, list)
-        refpkg = refpkg_dict[refpkg_name]  # type: ReferencePackage
-        unclassified_seqs[refpkg.prefix] = dict()
-        unclassified_seqs[refpkg.prefix]["low_lwr"] = list()
-        unclassified_seqs[refpkg.prefix]["big_pendant"] = list()
-        unclassified_seqs[refpkg.prefix]["svm"] = list()
+        ref_pkg = refpkg_dict[refpkg_name]  # type: refpkg.ReferencePackage
+        unclassified_seqs[ref_pkg.prefix] = dict()
+        unclassified_seqs[ref_pkg.prefix]["low_lwr"] = list()
+        unclassified_seqs[ref_pkg.prefix]["big_pendant"] = list()
+        unclassified_seqs[ref_pkg.prefix]["svm"] = list()
         svc_attempt = False
 
         for tree_sap in sorted(pqueries, key=lambda x: x.seq_name):  # type: phylo_seq.PQuery
             tree_sap.filter_min_weight_threshold(min_lwr)
             if not tree_sap.classified:
-                unclassified_seqs[refpkg.prefix]["low_lwr"].append(tree_sap)
+                unclassified_seqs[ref_pkg.prefix]["low_lwr"].append(tree_sap)
                 continue
             elif not tree_sap.placements:
                 continue
 
-            pplace = tree_sap.consensus_placement  # type: phylo_seq.PhyloPlace
+            tree_sap.avg_evo_dist = tree_sap.consensus_placement.total_distance()
+            tree_sap.string_distances()
 
-            leaf_children = tree_sap.node_map[int(pplace.edge_num)]
-
-            avg_tip_dist = round(pplace.mean_tip_length, 4)
-            pendant_length = round(pplace.pendant_length, 4)
-            distal_length = round(pplace.distal_length, 4)
-
-            tree_sap.avg_evo_dist = pplace.total_distance()
-            tree_sap.distances = ','.join([str(distal_length), str(pendant_length), str(avg_tip_dist)])
-
-            # hmm_perc = round((int(tree_sap.seq_len) * 100) / refpkg.profile_length, 1)
-
-            if pendant_length > max_pendant:
-                unclassified_seqs[refpkg.prefix]["big_pendant"].append(tree_sap)
+            if tree_sap.consensus_placement.pendant_length > max_pendant:
+                unclassified_seqs[ref_pkg.prefix]["big_pendant"].append(tree_sap)
                 tree_sap.classified = False
 
             if svc:
-                if refpkg.svc is None:
+                if ref_pkg.svc is None or len(features) != ref_pkg.svc.n_features_in_:
                     svc_attempt = True
                     call = 1
                 else:
-                    call = refpkg.svc.predict(preprocessing.normalize(np_array([len(leaf_children),
-                                                                                tree_sap.evalue,
-                                                                                round(pplace.like_weight_ratio, 2),
-                                                                                distal_length,
-                                                                                pendant_length,
-                                                                                avg_tip_dist]).reshape(1, -1)))
+                    classification_tup = training_utils.pquery_to_vector(tree_sap, ref_pkg)
+                    call = ref_pkg.svc.predict(np_array([getattr(classification_tup, feat) for feat in features]).reshape(1, -1))
                 # Discard this placement as a false positive if classifier calls this a 0
                 if call == 0:
                     unclassified_seqs[tree_sap.ref_name]["svm"].append(tree_sap)
                     tree_sap.classified = False
 
         if svc_attempt:
-            logging.warning("SVM classifier unavailable for reference package '{}'\n".format(refpkg.prefix))
+            logging.warning("SVM classifier unavailable for reference package '{}'\n".format(ref_pkg.prefix))
 
     logging.info("done.\n")
 
@@ -981,7 +986,7 @@ def select_query_placements(pquery_dict: dict, refpkg_dict: dict, mode="max_lwr"
     """
 
 
-    :return: Dictionary of PQuery instances indexed by denominator (refpkg code e.g. M0701)
+    :return: Dictionary of PQuery instances indexed by denominator (ref_pkg code e.g. M0701)
     """
 
     logging.info('Selecting the optimal query placements... ')
@@ -990,8 +995,8 @@ def select_query_placements(pquery_dict: dict, refpkg_dict: dict, mode="max_lwr"
     classified_seqs = 0
 
     for refpkg_code in pquery_dict:  # type: str
-        refpkg = refpkg_dict[refpkg_code]  # type: ReferencePackage
-        taxa_tree = refpkg.taxonomically_label_tree()
+        ref_pkg = refpkg_dict[refpkg_code]  # type: refpkg.ReferencePackage
+        taxa_tree = ref_pkg.taxonomically_label_tree()
         for pquery in pquery_dict[refpkg_code]:  # type: phylo_seq.PQuery
             if mode == "max_lwr":
                 pquery.process_max_weight_placement(taxa_tree)
@@ -1030,8 +1035,8 @@ def parse_raxml_output(epa_output_dir: str, refpkg_dict: dict, pqueries=None):
     :param refpkg_dict: A dictionary of ReferencePackage instances indexed by their prefix values
     :param pqueries: A list of instantiated PQuery instances
     :return:
-        1. Dictionary of PQuery instances indexed by denominator (refpkg code e.g. M0701)
-        2. Dictionary of an JPlace instance (values) mapped to a refpkg prefix.
+        1. Dictionary of PQuery instances indexed by denominator (ref_pkg code e.g. M0701)
+        2. Dictionary of an JPlace instance (values) mapped to a ref_pkg prefix.
          These instances store all PQuery instances from all JPlace files
     """
 
@@ -1049,9 +1054,9 @@ def parse_raxml_output(epa_output_dir: str, refpkg_dict: dict, pqueries=None):
         pquery_map = None
 
     for refpkg_name, jplace_list in jplace_utils.organize_jplace_files(jplace_files).items():
-        refpkg = refpkg_dict[refpkg_name]
-        if refpkg.prefix not in tree_saps:
-            tree_saps[refpkg.prefix] = list()
+        ref_pkg = refpkg_dict[refpkg_name]
+        if ref_pkg.prefix not in tree_saps:
+            tree_saps[ref_pkg.prefix] = list()
         for filename in jplace_list:
             # Load the JSON placement (jplace) file containing >= 1 pquery into JPlace object
             jplace_data = jplace_utils.jplace_parser(filename)
@@ -1062,21 +1067,21 @@ def parse_raxml_output(epa_output_dir: str, refpkg_dict: dict, pqueries=None):
             jplace_utils.calc_pquery_mean_tip_distances(jplace_data, internal_node_leaf_map)
             for pquery in jplace_data.pqueries:  # type: phylo_seq.PQuery
                 # Flesh out the internal-leaf node map
-                pquery.ref_name = refpkg.prefix
+                pquery.ref_name = ref_pkg.prefix
                 if not pquery.seq_name:
                     seq_info = re.match(r"(.*)\|" + re.escape(pquery.ref_name) + r"\|(\d+)_(\d+)$", pquery.place_name)
                     pquery.seq_name, pquery.start, pquery.end = seq_info.groups()
                 pquery.seq_len = int(pquery.end) - int(pquery.start)
                 pquery.node_map = internal_node_leaf_map
                 pquery.check_jplace_edge_lengths(edge_dist_index)
-                tree_saps[refpkg.prefix].append(pquery)
+                tree_saps[ref_pkg.prefix].append(pquery)
 
-            if refpkg.prefix not in itol_data:
-                itol_data[refpkg.prefix] = jplace_data
-                itol_data[refpkg.prefix].ref_name = refpkg.prefix
+            if ref_pkg.prefix not in itol_data:
+                itol_data[ref_pkg.prefix] = jplace_data
+                itol_data[ref_pkg.prefix].ref_name = ref_pkg.prefix
             else:
                 # If a JPlace file for that tree has already been parsed, just append the placements
-                itol_data[refpkg.prefix].pqueries = itol_data[refpkg.prefix].pqueries + jplace_data.pqueries
+                itol_data[ref_pkg.prefix].pqueries = itol_data[ref_pkg.prefix].pqueries + jplace_data.pqueries
 
             # I have decided to not remove the original JPlace files since some may find these useful
             # os.remove(filename)
@@ -1161,7 +1166,7 @@ def determine_confident_lineage(tree_saps: dict, refpkg_dict: dict, mode="max_lw
     leaf_taxa_map = dict()
     for refpkg_name in tree_saps:
         # All the leaves for that tree [number, translation, lineage]
-        ref_pkg = refpkg_dict[refpkg_name]  # type: ReferencePackage
+        ref_pkg = refpkg_dict[refpkg_name]  # type: refpkg.ReferencePackage
         for leaf in ref_pkg.generate_tree_leaf_references_from_refpkg():
             leaf_taxa_map[leaf.number] = leaf.lineage
 
@@ -1196,7 +1201,7 @@ def produce_itol_inputs(pqueries: dict, refpkg_dict: dict, jplaces: dict,
     Function to create outputs for the interactive tree of life (iTOL) webservice.
     There is a directory for each of the marker genes detected to allow the user to "drag-and-drop" all files easily
 
-    :param pqueries: Dictionary of PQuery instances indexed by denominator (refpkg code e.g. M0701)
+    :param pqueries: Dictionary of PQuery instances indexed by denominator (ref_pkg code e.g. M0701)
     :param refpkg_dict: A dictionary of ReferencePackage instances indexed by their prefix values
     :param jplaces: Dictionary of an JPlace instance (values) mapped to marker name
     :param itol_base_dir: Output directory to write the iTOL_output files, with the outputs stored within a directory
@@ -1216,38 +1221,38 @@ def produce_itol_inputs(pqueries: dict, refpkg_dict: dict, jplaces: dict,
         if len(pqueries[refpkg_name]) == 0:
             # No sequences that were mapped met the minimum likelihood weight ration threshold. Skipping!
             continue
-        refpkg = refpkg_dict[refpkg_name]  # type: ReferencePackage
-        if not os.path.exists(itol_base_dir + refpkg.prefix):
-            os.mkdir(itol_base_dir + refpkg.prefix)
-        jplace_data = jplaces[refpkg.prefix]
+        ref_pkg = refpkg_dict[refpkg_name]  # type: refpkg.ReferencePackage
+        if not os.path.exists(itol_base_dir + ref_pkg.prefix):
+            os.mkdir(itol_base_dir + ref_pkg.prefix)
+        jplace_data = jplaces[ref_pkg.prefix]
         refpkg_pqueries = pqueries[refpkg_name]
 
-        if os.path.isfile(refpkg.f__boot_tree):
-            jplace_utils.add_bipartitions(jplace_data, refpkg.f__boot_tree)
+        if os.path.isfile(ref_pkg.f__boot_tree):
+            jplace_utils.add_bipartitions(jplace_data, ref_pkg.f__boot_tree)
 
         # Make a master jplace file from the set of placements in all jplace files for each marker
-        master_jplace = os.path.join(itol_base_dir, refpkg.prefix, refpkg.prefix + "_complete_profile.jplace")
+        master_jplace = os.path.join(itol_base_dir, ref_pkg.prefix, ref_pkg.prefix + "_complete_profile.jplace")
         # TODO: validate no distal lengths exceed their corresponding edge lengths
 
         jplace_data.write_jplace(master_jplace)
-        jplaces[refpkg.prefix].clear_object()
+        jplaces[ref_pkg.prefix].clear_object()
         jplace_data.clear_object()
         # Create a labels file from the tax_ids_marker.txt
-        refpkg.create_itol_labels(itol_base_dir)
+        ref_pkg.create_itol_labels(itol_base_dir)
 
-        annotation_style_files = glob.glob(os.sep.join([treesapp_data_dir, "iTOL_data", refpkg.prefix + "*"]))
+        annotation_style_files = glob.glob(os.sep.join([treesapp_data_dir, "iTOL_data", ref_pkg.prefix + "*"]))
         # Copy the respective colours and styles files for each marker found to the itol_output directories
-        colours_styles = os.sep.join([treesapp_data_dir, "iTOL_data", refpkg.prefix + "_colours_style.txt"])
-        colour_strip = os.sep.join([treesapp_data_dir, "iTOL_data", refpkg.prefix + "_colour_strip.txt"])
+        colours_styles = os.sep.join([treesapp_data_dir, "iTOL_data", ref_pkg.prefix + "_colours_style.txt"])
+        colour_strip = os.sep.join([treesapp_data_dir, "iTOL_data", ref_pkg.prefix + "_colour_strip.txt"])
         if colours_styles not in annotation_style_files:
-            style_missing.append(refpkg.prefix)
+            style_missing.append(ref_pkg.prefix)
         if colour_strip not in annotation_style_files:
-            strip_missing.append(refpkg.prefix)
+            strip_missing.append(ref_pkg.prefix)
 
         for annotation_file in annotation_style_files:
-            shutil.copy(annotation_file, itol_base_dir + refpkg.prefix)
-        itol_bar_file = os.path.join(itol_base_dir, refpkg.prefix, refpkg.prefix + "_abundance_simplebar.txt")
-        generate_simplebar(refpkg.prefix, refpkg_pqueries, itol_bar_file)
+            shutil.copy(annotation_file, itol_base_dir + ref_pkg.prefix)
+        itol_bar_file = os.path.join(itol_base_dir, ref_pkg.prefix, ref_pkg.prefix + "_abundance_simplebar.txt")
+        abundance.generate_simplebar(ref_pkg.prefix, refpkg_pqueries, itol_bar_file)
 
     logging.info("done.\n")
     if style_missing:
@@ -1260,6 +1265,24 @@ def produce_itol_inputs(pqueries: dict, refpkg_dict: dict, jplaces: dict,
     return
 
 
+def alert_for_refpkg_feature_annotations(pqueries: dict, refpkg_dict: dict) -> None:
+    feature_positive = []
+    for refpkg_name in pqueries:
+        if len(pqueries[refpkg_name]) == 0:
+            continue
+        ref_pkg = refpkg_dict[refpkg_name]  # type: refpkg.ReferencePackage
+        if len(ref_pkg.feature_annotations) > 0:
+            feature_positive.append(ref_pkg.prefix)
+
+    if len(feature_positive) > 0:
+        logging.info("Alert: {} reference package(s) have feature annotations. "
+                     "Consider running treesapp layer.\n".format(len(feature_positive)))
+        logging.debug("Reference packages with clade_annotations attribute filled:\n\t{}\n"
+                      "".format("\n\t".join(feature_positive)))
+
+    return
+
+
 def assign(sys_args):
     # STAGE 1: Prompt the user and prepare files and lists for the pipeline
     parser = treesapp_args.TreeSAPPArgumentParser(description='Classify sequences through evolutionary placement.')
@@ -1280,9 +1303,10 @@ def assign(sys_args):
     ts_assign.decide_stage(args)
     n_proc = args.num_threads
 
-    refpkg_dict = file_parsers.gather_ref_packages(ts_assign.refpkg_dir, ts_assign.target_refpkgs)
+    refpkg_dict = refpkg.gather_ref_packages(ts_assign.refpkg_dir, ts_assign.target_refpkgs)
     prep_reference_packages_for_assign(refpkg_dict, ts_assign.var_output_dir)
     ref_alignment_dimensions = get_alignment_dims(refpkg_dict)
+    ts_assign.load_refpkg_classifiers(refpkg_dict, kernel=args.kernel, threads=n_proc)
 
     ##
     # STAGE 2: Predict open reading frames (ORFs) if the input is an assembly, read, format and write the FASTA
@@ -1413,6 +1437,8 @@ def assign(sys_args):
         produce_itol_inputs(tree_saps, refpkg_dict, itol_data, ts_assign.itol_out, ts_assign.refpkg_dir)
         delete_files(args.delete, ts_assign.stage_lookup("place").dir_path, 4)
 
+        alert_for_refpkg_feature_annotations(tree_saps, refpkg_dict)
+
     # Clear out the rest of the intermediates
     delete_files(args.delete, ts_assign.var_output_dir, 5)
 
