@@ -82,13 +82,13 @@ def info(sys_args):
     logging.info(utilities.executable_dependency_versions(ts_info.executables))
 
     if args.verbose:
-        refpkg_dict = file_parsers.gather_ref_packages(ts_info.refpkg_dir)
-        refpkg_summary_str = "\t".join(["Name", "Code-name",
+        refpkg_dict = ts_ref_pkg.gather_ref_packages(ts_info.refpkg_dir)
+        refpkg_summary_str = "\t".join(["Name",
                                         "Molecule", "Tree builder", "RefPkg-type", "Leaf nodes",
                                         "Description", "Created", "Last-updated"]) + "\n"
         for refpkg_name in sorted(refpkg_dict, key=lambda x: refpkg_dict[x].prefix):
             refpkg = refpkg_dict[refpkg_name]  # type: ts_ref_pkg.ReferencePackage
-            refpkg_summary_str += "\t".join([refpkg_name, refpkg.refpkg_code,
+            refpkg_summary_str += "\t".join([refpkg_name,
                                              refpkg.molecule, refpkg.tree_tool, refpkg.kind, str(refpkg.num_seqs),
                                              refpkg.description, refpkg.date, refpkg.update]
                                             ) + "\n"
@@ -123,26 +123,36 @@ Use '-h' to get subcommand-specific help, e.g. 'treesapp package view -h'
         sys.stderr.write(pkg_usage)
         sys.exit(1)
 
-    refpkg = ts_ref_pkg.ReferencePackage()
+    ref_pkg = ts_ref_pkg.ReferencePackage()
 
-    treesapp_args.add_package_arguments(parser, refpkg.get_public_attributes())
+    treesapp_args.add_package_arguments(parser, ref_pkg.get_public_attributes())
     args = parser.parse_args(sys_args)
 
-    if not os.path.isdir(args.output):
-        os.mkdir(args.output)
+    if args.output:
+        log_dir = args.output
+        if not os.path.isdir(args.output):
+            os.mkdir(args.output)
+    else:
+        log_dir = "./"
 
-    classy.prep_logging(log_file=os.path.join(args.output, 'TreeSAPP_package_log.txt'))
+    classy.prep_logging(log_file=os.path.join(log_dir, 'TreeSAPP_package_log.txt'))
 
     for refpkg_pkl in args.pkg_path:
-        refpkg.f__json = refpkg_pkl
-        refpkg.slurp()
+        ref_pkg.f__pkl = refpkg_pkl
+        ref_pkg.slurp()
+
+        if args.output:
+            output_dir = args.output
+        else:
+            output_dir = os.path.dirname(ref_pkg.f__pkl)
 
         if args.subcommand == "view":
-            ts_ref_pkg.view(refpkg, args.attributes)
+            ts_ref_pkg.view(ref_pkg, args.attributes)
         elif args.subcommand == "edit":
-            ts_ref_pkg.edit(refpkg, args.attributes, args.output, args.overwrite)
+            ts_ref_pkg.edit(ref_pkg, args.attributes, output_dir,
+                            overwrite=args.overwrite, phenotypes=args.phenotypes, reset=args.reset, join=args.join)
         elif args.subcommand == "rename":
-            ts_ref_pkg.rename(refpkg, args.attributes, args.output, args.overwrite)
+            ts_ref_pkg.rename(ref_pkg, args.attributes, output_dir, args.overwrite)
         else:
             logging.error("Unrecognized command: '{}'.\n{}\n".format(args.subcommand, pkg_usage))
             sys.exit(1)
@@ -155,7 +165,7 @@ def train(sys_args):
     treesapp_args.add_trainer_arguments(parser)
     args = parser.parse_args(sys_args)
 
-    ts_trainer = classy.PhyTrainer()
+    ts_trainer = training_utils.PhyTrainer()
     ts_trainer.furnish_with_arguments(args)
     ts_trainer.check_previous_output(args.overwrite)
 
@@ -164,7 +174,7 @@ def train(sys_args):
     logging.info("\n##\t\t\tTrain taxonomic rank-placement distance model\t\t\t##\n")
 
     treesapp_args.check_parser_arguments(args, sys_args)
-    treesapp_args.check_trainer_arguments(ts_trainer, args)
+    ts_trainer.check_trainer_arguments(args)
     ts_trainer.set_file_paths()
     ts_trainer.decide_stage(args)
     ts_trainer.ref_pkg.disband(os.path.join(ts_trainer.var_output_dir, ts_trainer.ref_pkg.prefix + "_RefPkg"))
@@ -225,7 +235,8 @@ def train(sys_args):
     ##
     # STAGE 3: Download the taxonomic lineages for each query sequence
     ##
-    entrez_record_dict = ts_trainer.fetch_entrez_lineages(train_seqs, args.molecule,
+    entrez_record_dict = ts_trainer.fetch_entrez_lineages(train_seqs,
+                                                          molecule=ts_trainer.ref_pkg.molecule,
                                                           acc_to_taxid=args.acc_to_taxid,
                                                           seqs_to_lineage=args.seq_names_to_taxa)
 
@@ -276,7 +287,7 @@ def train(sys_args):
         assign_params = ["-i", train_seqs.file,
                          "-o", assign_prefix,
                          "--num_procs", str(args.num_threads),
-                         "--refpkg_dir", os.path.dirname(ts_trainer.ref_pkg.f__json),
+                         "--refpkg_dir", os.path.dirname(ts_trainer.ref_pkg.f__pkl),
                          "--targets", ts_trainer.ref_pkg.prefix,
                          "--molecule", ts_trainer.ref_pkg.molecule,
                          "--delete"]
@@ -286,7 +297,7 @@ def train(sys_args):
         try:
             ts_assign_mod.assign(assign_params)
             plain_pqueries = ts_phylo_seq.assignments_to_pqueries(file_parsers.read_classification_table(
-                os.path.join(assign_prefix, "final_outputs", "marker_contig_map.tsv")))
+                os.path.join(assign_prefix, "final_outputs", "classifications.tsv")))
         except (SystemExit, IOError):
             logging.info("failed.\n")
             logging.error("treesapp assign did not complete successfully.\n")
@@ -338,26 +349,18 @@ def train(sys_args):
                 fp_names[refpkg_name] = [qseq.seq_name for qseq in fp_query_seqs]
 
         logging.info("Extracting features from TreeSAPP classifications... ")
-        tp, t_pqs = training_utils.vectorize_placement_data(condition_names=tp_names, classifieds=refpkg_pqueries,
-                                                            refpkg_map={ts_trainer.ref_pkg.prefix: ts_trainer.ref_pkg})
-        fp, f_pqs = training_utils.vectorize_placement_data(condition_names=fp_names, classifieds=refpkg_pqueries,
-                                                            refpkg_map={ts_trainer.ref_pkg.prefix: ts_trainer.ref_pkg})
+        training_df = training_utils.load_training_data_frame(pqueries=refpkg_pqueries,
+                                                              refpkg_map={ts_trainer.ref_pkg.prefix:
+                                                                          ts_trainer.ref_pkg},
+                                                              refpkg_positive_annots=tp_names)
         logging.info("done.\n")
-
-        # Split the training and testing data from the classifications
-        classified_data, conditions, x_train, x_test, y_train, y_test = training_utils.generate_train_test_data(tp, fp)
-        # Save the feature vectors and condition vectors
-        training_utils.save_np_array(classified_data, ts_trainer.feature_vector_file)
-        training_utils.save_np_array(conditions, ts_trainer.conditions_file)
-        # Train the classifier
-        refpkg_classifiers = training_utils.train_classification_filter(tp_names, t_pqs, classified_data, conditions,
-                                                                        x_train, x_test, y_train, y_test,
-                                                                        kernel=args.kernel, tsne=ts_trainer.tsne_plot,
-                                                                        grid_search=args.grid_search,
-                                                                        num_procs=args.num_threads)
+        training_utils.train_classifier_from_dataframe(training_df, args.kernel,
+                                                       grid_search=args.grid_search,
+                                                       num_threads=args.num_threads,
+                                                       tsne=ts_trainer.tsne_plot)
         ts_trainer.increment_stage_dir()
     else:
-        refpkg_classifiers = {}
+        training_df = training_utils.load_training_data_frame({}, {}, {})
 
     if ts_trainer.stage_status("update"):
         if not ts_trainer.ref_pkg.pfit:
@@ -365,9 +368,9 @@ def train(sys_args):
                             "Taxonomic ranks will not be distance-adjusted during classification for this package.\n")
             ts_trainer.ref_pkg.pfit = [0.0, 7.0]
 
-        ts_trainer.ref_pkg.svc = refpkg_classifiers[ts_trainer.ref_pkg.prefix]
-        ts_trainer.ref_pkg.f__json = os.path.join(ts_trainer.final_output_dir,
-                                                  os.path.basename(ts_trainer.ref_pkg.f__json))
+        ts_trainer.ref_pkg.training_df = training_df
+        ts_trainer.ref_pkg.f__pkl = os.path.join(ts_trainer.final_output_dir,
+                                                 os.path.basename(ts_trainer.ref_pkg.f__pkl))
 
         ts_trainer.ref_pkg.validate()
         ts_trainer.ref_pkg.pickle_package()
@@ -402,7 +405,7 @@ def create(sys_args):
     logging.info("\n##\t\t\tCreating TreeSAPP reference package\t\t\t##\n")
 
     treesapp_args.check_parser_arguments(args, sys_args)
-    treesapp_args.check_create_arguments(ts_create, args)
+    ts_create_mod.check_create_arguments(ts_create, args)
     ts_create.decide_stage(args)
 
     # Populate the final TreeSAPP reference file paths with the proper output directory
@@ -703,8 +706,8 @@ def create(sys_args):
     # Finish validating the file and append the reference package build parameters to the master table
     ##
     if ts_create.stage_status("update"):
-        ts_create.ref_pkg.f__json = os.path.join(ts_create.var_output_dir, "placement_trainer", "final_outputs",
-                                                 ts_create.ref_pkg.prefix + ts_create.ref_pkg.refpkg_suffix)
+        ts_create.ref_pkg.f__pkl = os.path.join(ts_create.var_output_dir, "placement_trainer", "final_outputs",
+                                                ts_create.ref_pkg.prefix + ts_create.ref_pkg.refpkg_suffix)
         ts_create.ref_pkg.slurp()
         ts_create.ref_pkg.validate()
         ts_create.ref_pkg.change_file_paths(ts_create.final_output_dir)
@@ -741,11 +744,12 @@ def update(sys_args):
     classified_fasta = fasta.FASTA(ts_updater.query_sequences)  # These are the classified sequences
     classified_fasta.load_fasta()
     classified_fasta.add_accession_to_headers(ts_updater.ref_pkg.prefix)
-    classified_lines = file_parsers.read_classification_table(ts_updater.assignment_table)
-    candidate_update_seqs = ts_update_mod.filter_by_lwr(classified_lines, args.min_lwr)
-    classified_targets = utilities.match_target_marker(ts_updater.ref_pkg.prefix, classified_fasta.get_seq_names())
-    name_map = ts_update_mod.strip_assigment_pattern(classified_fasta.get_seq_names(), ts_updater.ref_pkg.prefix)
-    classified_targets = ts_update_mod.intersect_incomparable_lists(classified_targets, candidate_update_seqs, name_map)
+    pqueries = file_parsers.load_classified_sequences_from_assign_output(assign_output_dir=ts_updater.treesapp_output,
+                                                                         assigner_cls=ts_assign_mod.Assigner(),
+                                                                         refpkg_name=ts_updater.ref_pkg.prefix)
+    candidate_update_pqueries = ts_update_mod.filter_by_placement_thresholds(pqueries,
+                                                                             args.min_lwr, args.max_pd, args.max_evo)
+    classified_targets = [pq.place_name for pq in candidate_update_pqueries]
     # Remove classified sequences that are already in the reference package
     ts_update_mod.drop_queries_by_accession(classified_targets, ref_seq_lineage_info)
 
@@ -777,7 +781,8 @@ def update(sys_args):
                                                          ts_updater.ref_pkg.prefix)
         querying_classified_fasta.synchronize_seqs_n_headers()
         querying_classified_fasta.swap_headers(name_map)
-        fasta_records = ts_updater.fetch_entrez_lineages(ref_seqs=querying_classified_fasta, molecule=args.molecule,
+        fasta_records = ts_updater.fetch_entrez_lineages(ref_seqs=querying_classified_fasta,
+                                                         molecule=ts_updater.ref_pkg.molecule,
                                                          seqs_to_lineage=ts_updater.seq_names_to_taxa)
         entrez_utils.fill_ref_seq_lineages(fasta_records, classified_seq_lineage_map)
         ref_leaf_nodes = ts_phylo_seq.convert_entrez_to_tree_leaf_references(fasta_records)
@@ -791,10 +796,9 @@ def update(sys_args):
         querying_classified_fasta.synchronize_seqs_n_headers()
     else:
         # Map candidate reference sequence names to their TreeSAPP-assigned taxonomies
-        assignments = file_parsers.parse_assignments(classified_lines)
-        classified_seq_lineage_map.update(ts_update_mod.map_classified_seqs(ts_updater.ref_pkg.prefix,
-                                                                            assignments,
-                                                                            querying_classified_fasta.get_seq_names()))
+        classified_seq_lineage_map.update(
+            {pq.place_name.split(' ')[0]: pq.recommended_lineage for pq in candidate_update_pqueries}
+        )
 
     ref_header_map = {leaf.number + '_' + ts_updater.ref_pkg.prefix: leaf.description for leaf in ref_seq_lineage_info}
     ref_header_map = ts_update_mod.reformat_ref_seq_descriptions(ref_header_map)
@@ -918,7 +922,7 @@ def update(sys_args):
     ##
     create_cmd = ts_update_mod.formulate_create_command(ts_updater, args, final_stage="support")
     create(create_cmd)
-    ts_updater.updated_refpkg.f__json = ts_updater.updated_refpkg_path
+    ts_updater.updated_refpkg.f__pkl = ts_updater.updated_refpkg_path
     ts_updater.updated_refpkg.slurp()
 
     if ts_updater.stage_status("train"):
@@ -926,26 +930,29 @@ def update(sys_args):
                                                     ref_pkg=ts_updater.updated_refpkg,
                                                     output_dir=ts_updater.training_dir,
                                                     args=args))
-        ts_updater.updated_refpkg.f__json = os.path.join(ts_updater.training_dir, "final_outputs",
-                                                         ts_updater.ref_pkg.prefix + ts_updater.ref_pkg.refpkg_suffix)
+        ts_updater.updated_refpkg.f__pkl = os.path.join(ts_updater.training_dir, "final_outputs",
+                                                        ts_updater.ref_pkg.prefix + ts_updater.ref_pkg.refpkg_suffix)
         ts_updater.updated_refpkg.slurp()
     else:
+        ts_updater.updated_refpkg.training_df = ts_updater.ref_pkg.training_df
         ts_updater.updated_refpkg.pfit = ts_updater.ref_pkg.pfit
         ts_updater.updated_refpkg.svc = ts_updater.ref_pkg.svc
 
+    ts_update_mod.update_features(old_refpkg=ts_updater.ref_pkg, new_refpkg=ts_updater.updated_refpkg)
+
     ##
     # Summarize some key parts of the new reference package, compared to the old one
     ##
     ts_updater.updated_refpkg.validate()
-    ts_updater.ref_pkg.change_file_paths(ts_updater.final_output_dir)
-    ts_updater.update_refpkg_fields()
+    ts_updater.update_refpkg_fields(output_dir=os.path.dirname(ts_updater.updated_refpkg_path))
 
     return
 
 
 def colour(sys_args):
-    parser = treesapp_args.TreeSAPPArgumentParser(description="Colours a reference package's phylogeny based on"
-                                                              " taxonomic or phenotypic data.")
+    parser = treesapp_args.TreeSAPPArgumentParser(description="Generates colour style and strip files for visualizing "
+                                                              "a reference package's phylogeny in iTOL based on "
+                                                              "taxonomic or phenotypic data.")
     treesapp_args.add_colour_arguments(parser)
     args = parser.parse_args(sys_args)
 
@@ -958,16 +965,18 @@ def colour(sys_args):
     ts_painter = paint.PhyPainter()
     ts_painter.primer(args)
 
-    if ts_painter.phenotypes:
-        ts_painter.taxa_phenotype_map = paint.read_phenotypes(args.phenotypes)
-
     # Find the taxa that should be coloured for each reference package
     for refpkg_name, ref_pkg in ts_painter.refpkg_dict.items():  # type: (str, ts_ref_pkg.ReferencePackage)
-        if ts_painter.phenotypes:
-            target_taxa = list(ts_painter.taxa_phenotype_map.keys())
-            taxon_leaf_map = paint.convert_taxa_to_phenotypes(taxon_leaf_map=paint.map_taxa_to_leaf_nodes(target_taxa,
-                                                                                                          ref_pkg),
-                                                              phenotypes_map=ts_painter.taxa_phenotype_map)
+        if args.attribute != 'taxonomy':
+            taxon_leaf_map = {}
+            try:
+                clade_annots = ref_pkg.feature_annotations[ts_painter.feature_name]
+            except KeyError:
+                logging.warning("Reference package '{}' doesn't have the '{}' feature annotated. "
+                                "It is being skipped\n".format(refpkg_name, ts_painter.feature_name))
+                continue
+            for ca in clade_annots:
+                taxon_leaf_map.update({ca.name: ca.members})
         else:
             ts_painter.find_rank_depth(ref_pkg, ref_pkg.taxa_trie.accepted_ranks_depths[ts_painter.rank])
 
@@ -992,22 +1001,26 @@ def colour(sys_args):
         # Find the intersection or union between reference packages analyzed so far
         ts_painter.harmonize_taxa_colours(taxon_leaf_map, args.set_op)
 
+    if len(ts_painter.refpkg_leaf_nodes_to_colour.keys()) == 0:
+        logging.error("Unable to colour phylogenies by '{}' - attributes were not found in reference packages.\n".format(args.attribute))
+        raise AssertionError
+
     # Sort the nodes by their internal node order
     taxa_order = paint.order_taxa(taxa_to_colour=ts_painter.taxa_to_colour,
                                   taxon_leaf_map=ts_painter.refpkg_leaf_nodes_to_colour[ref_pkg.prefix],
                                   leaf_order=ref_pkg.leaf_node_order())
 
     # Determine the palette to use for taxa across all reference packages
-    colours = paint.get_colours(ts_painter.taxa_to_colour, ts_painter.palette, ts_painter.rank)
+    colours = ts_painter.get_colours()
     palette_taxa_map = paint.map_colours_to_taxa(taxa_order, colours)
 
     # Create the iTOL colour files
     for refpkg_name, ref_pkg in ts_painter.refpkg_dict.items():  # type: (str, ts_ref_pkg.ReferencePackage)
         taxon_leaf_map = ts_painter.refpkg_leaf_nodes_to_colour[refpkg_name]
         style_file = os.path.join(ts_painter.output_dir,
-                                  "{}_{}_colours_style.txt".format(ref_pkg.prefix, ts_painter.output_prefix))
+                                  "{}_{}_colours_style.txt".format(ref_pkg.prefix, ts_painter.feature_name))
         strip_file = os.path.join(ts_painter.output_dir,
-                                  "{}_{}_colour_strip.txt".format(ref_pkg.prefix, ts_painter.output_prefix))
+                                  "{}_{}_colour_strip.txt".format(ref_pkg.prefix, ts_painter.feature_name))
 
         # Find the minimum set of monophyletic internal nodes for each taxon
         taxa_clades = ts_painter.find_mono_clades(taxon_leaf_map, ref_pkg)
@@ -1022,71 +1035,58 @@ def colour(sys_args):
 
 def layer(sys_args):
     # STAGE 1: Prompt the user and prepare files and lists for the pipeline
-    parser = treesapp_args.TreeSAPPArgumentParser(description="This script is generally used for layering extra "
-                                                              "annotations such as Subgroup or Metabolism on TreeSAPP "
-                                                              "outputs. This is accomplished by adding an extra column "
-                                                              "(to all rows) of an existing marker_contig_map.tsv and "
-                                                              "annotating the relevant sequences.")
+    parser = treesapp_args.TreeSAPPArgumentParser(description="This script adds extra feature annotations, such as "
+                                                              "Subgroup and Metabolic Pathway, to an existing "
+                                                              "classification table made by treesapp assign. "
+                                                              "A new column is bound to the table for each feature.")
     treesapp_args.add_layer_arguments(parser)
     args = parser.parse_args(sys_args)
 
-    ts_layer = classy.Layerer()
+    ts_layer = annotate_extra.Layerer()
 
     log_file_name = args.output + os.sep + "TreeSAPP_layer_log.txt"
     classy.prep_logging(log_file_name, args.verbose)
     logging.info("\n##\t\t\t\tLayering extra annotations on TreeSAPP classifications\t\t\t\t##\n\n")
 
-    annotate_extra.check_arguments(ts_layer, args)
+    ts_layer.check_arguments(args)
 
     ##
     # Worklow:
     #   1. Slurp up reference packages into a dictionary
-    #   2. Read the marker_contig_map.tsv file from the output directory to create the master data structure
+    #   2. Read the classifications.tsv file from the output directory to create the master data structure
     #   3. For each of the colours_styles files provided (potentially multiple for the same marker):
     #       3.1) Add the annotation variable to master_dat for every sequence (instantiate with "NA")
     #       3.2) Read the .jplace file for every sequence classified as marker
     #       3.3) Add the annotation information to every sequence classified as marker in master_dat
-    #   4. Write the new classification file called "extra_annotated_marker_contig_map.tsv"
+    #   4. Write the new classification file called "layered_classifications.tsv"
     ##
-    marker_subgroups = dict()
+    marker_subgroups = set()
     unique_markers_annotated = set()
     marker_tree_info = dict()
     master_dat, field_order = annotate_extra.parse_marker_classification_table(ts_layer.final_output_dir +
-                                                                               "marker_contig_map.tsv")
-    refpkg_dict = file_parsers.gather_ref_packages(ts_layer.refpkg_dir)
+                                                                               ts_layer.classification_tbl_name)
+    refpkg_dict = ts_ref_pkg.gather_ref_packages(ts_layer.refpkg_dir)
 
     # structure of master dat:
     # {"Sequence_1": {"Field1": x, "Field2": y, "Extra": n},
     #  "Sequence_2": {"Field1": i, "Field2": j, "Extra": n}}
-    for annot_f in ts_layer.annot_files:
-        # Determine the marker being annotated
-        data_type, refpkg_prefix = "", ""
-        for refpkg_name in refpkg_dict:  # type: str
-            annot_marker_re = re.compile(r"^{0}_(\w+)_colours_style.txt$".format(refpkg_name))
-            if annot_marker_re.match(os.path.basename(annot_f)):
-                data_type = annot_marker_re.match(os.path.basename(annot_f)).group(1)
-                refpkg_prefix = refpkg_name
-                break
-            else:
-                data_type, refpkg_prefix = "", ""
+    for refpkg_prefix, ref_pkg in refpkg_dict.items():  # type: (str, ts_ref_pkg.ReferencePackage)
         if refpkg_prefix not in master_dat.keys():
             continue
-        if refpkg_prefix and data_type:
+        if len(ref_pkg.feature_annotations) > 0:
             unique_markers_annotated.add(refpkg_prefix)
-            if data_type not in marker_subgroups:
-                marker_subgroups[data_type] = dict()
-            marker_subgroups[data_type][refpkg_prefix] = file_parsers.read_colours_file(annot_f, refpkg_prefix)
-        else:
-            logging.warning("Unable to parse the reference package name and/or annotation type from {}.\n"
-                            "Is it possible this reference package is not in {}?".format(annot_f, ts_layer.refpkg_dir))
-    # Instantiate every query sequence in marker_contig_map with an empty string for each data_type
+            for data_type in ref_pkg.feature_annotations:
+                marker_subgroups.add(data_type)
+
+    # Instantiate every query sequence in classifications with an empty string for each data_type
     for data_type in marker_subgroups:
         for refpkg_name in master_dat:
             for assignment in master_dat[refpkg_name]:  # type: annotate_extra.ClassifiedSequence
                 assignment.layers[data_type] = "NA"
+
     # Update the field_order dictionary with new fields
     field_acc = len(field_order)
-    for new_datum in sorted(marker_subgroups.keys()):
+    for new_datum in sorted(marker_subgroups):
         field_order[field_acc] = new_datum
         field_acc += 1
 
@@ -1098,21 +1098,18 @@ def layer(sys_args):
         for refpkg_name in unique_markers_annotated:  # type: str
             jplace = os.path.join(ts_layer.treesapp_output, "iTOL_output", refpkg_name,
                                   refpkg_name + "_complete_profile.jplace")
-
-            if refpkg_name in marker_subgroups[data_type]:
-                refpkg = refpkg_dict[refpkg_name]  # type: ts_ref_pkg.ReferencePackage
+            ref_pkg = refpkg_dict[refpkg_name]  # type: ts_ref_pkg.ReferencePackage
+            if data_type in ref_pkg.feature_annotations:
                 logging.info("\t" + refpkg_name + "\n")
                 # Create the dictionary mapping an internal node to all leaves
                 internal_node_map = entish.map_internal_nodes_leaves(jplace_utils.jplace_parser(jplace).tree)
                 # Routine for exchanging any organism designations for their respective node number
-                taxa_map = refpkg.generate_tree_leaf_references_from_refpkg()
+                taxa_map = ref_pkg.generate_tree_leaf_references_from_refpkg()
 
-                clusters = annotate_extra.names_for_nodes(marker_subgroups[data_type][refpkg_name],
-                                                          internal_node_map, taxa_map)
-                clusters = utilities.convert_outer_to_inner_nodes(clusters, internal_node_map)
+                annotated_edges, leaves_in_clusters = annotate_extra.annotate_internal_nodes(internal_node_map,
+                                                                                             ref_pkg.feature_annotations[data_type])
+                marker_tree_info[data_type][refpkg_name] = annotated_edges
 
-                marker_tree_info[data_type][refpkg_name], leaves_in_clusters = annotate_extra.annotate_internal_nodes(internal_node_map,
-                                                                                                                 clusters)
                 diff = len(taxa_map) - len(leaves_in_clusters)
                 if diff != 0:
                     unannotated = set()
@@ -1120,13 +1117,15 @@ def layer(sys_args):
                         for leaf in internal_node_map[inode]:
                             if leaf not in leaves_in_clusters:
                                 unannotated.add(str(leaf))
-                    logging.warning("The following leaf nodes were not mapped to annotation groups:\n" +
-                                    "\t" + ', '.join(sorted(unannotated, key=lambda x: int(x.split('_')[0]))) + "\n")
+                    logging.warning("{} leaf nodes were not mapped to annotation groups. "
+                                    "More information can be found in the log.\n".format(len(unannotated)))
+                    logging.debug("The following leaf nodes were not mapped to annotation groups:\n" +
+                                  "\t" + ', '.join(sorted(unannotated, key=lambda x: int(x.split('_')[0]))) + "\n")
             else:
                 pass
     marker_subgroups.clear()
     master_dat = annotate_extra.map_queries_to_annotations(marker_tree_info, master_dat)
-    annotate_extra.write_classification_table(ts_layer.final_output_dir, field_order, master_dat)
+    annotate_extra.write_classification_table(ts_layer.layered_table, field_order, master_dat)
 
     return
 
@@ -1183,9 +1182,10 @@ def purity(sys_args):
             assigned_lines = file_parsers.read_classification_table(ts_purity.classifications)
             ts_purity.assignments = file_parsers.parse_assignments(assigned_lines)
         else:
-            logging.error("marker_contig_map.tsv is missing from output directory '" +
-                          os.path.dirname(ts_purity.classifications) + "'\n" +
-                          "Please remove this directory and re-run.\n")
+            logging.error("{} is missing from output directory '{}'\n"
+                          "Please remove this directory and re-run.\n"
+                          "".format(ts_purity.classification_tbl_name,
+                                    os.path.dirname(ts_purity.classifications)))
             sys.exit(5)
 
         logging.info("\nSummarizing assignments for reference package " + ts_purity.ref_pkg.prefix + "\n")
@@ -1241,12 +1241,12 @@ def evaluate(sys_args):
     ref_lineages = {leaf.number: leaf.lineage for leaf in ref_leaves}
 
     # Load FASTA data
-    query_fasta = fasta.FASTA(args.input)
-    query_fasta.load_fasta(format_it=True, molecule=args.molecule)
+    query_fasta = fasta.FASTA(ts_evaluate.input_sequences)
+    query_fasta.load_fasta(format_it=True, molecule=ts_evaluate.molecule_type)
     if args.length:
         query_fasta.trim_to_length(args.length)
 
-    fasta_records = ts_evaluate.fetch_entrez_lineages(query_fasta, args.molecule, args.acc_to_taxid)
+    fasta_records = ts_evaluate.fetch_entrez_lineages(query_fasta, ts_evaluate.molecule_type, args.acc_to_taxid)
     entrez_utils.fill_ref_seq_lineages(fasta_records, ts_evaluate.seq_lineage_map)
     query_leaf_nodes = ts_phylo_seq.convert_entrez_to_tree_leaf_references(fasta_records)
     ts_evaluate.ref_pkg.taxa_trie.feed_leaf_nodes(query_leaf_nodes)
