@@ -6,7 +6,9 @@ __maintainer__ = "Connor Morgan-Lang"
 import logging
 import sys
 import re
+import os
 
+from treesapp import seq_clustering
 from treesapp.wrapper import run_odseq, run_mafft
 from treesapp.lca_calculations import megan_lca, clean_lineage_list
 from treesapp.taxonomic_hierarchy import TaxonomicHierarchy
@@ -15,6 +17,83 @@ from treesapp import fasta
 from treesapp import classy
 
 
+def check_create_arguments(creator: classy.Creator, args) -> None:
+    creator.combine_input_files()
+    creator.find_sequence_molecule_type()
+    # Populate ReferencePackage attributes from command-line arguments
+    if args.fast:
+        creator.ref_pkg.tree_tool = "FastTree"
+    else:
+        creator.ref_pkg.tree_tool = "RAxML-NG"
+    creator.ref_pkg.prefix = args.refpkg_name
+    creator.ref_pkg.pid = args.similarity
+    creator.ref_pkg.molecule = creator.molecule_type
+    creator.ref_pkg.kind = args.kind
+    creator.ref_pkg.sub_model = args.raxml_model
+    creator.ref_pkg.f__pkl = creator.final_output_dir + creator.ref_pkg.prefix + creator.ref_pkg.refpkg_suffix
+    # TODO: Create placement trainer output directory and make it an attribute
+    if not args.output:
+        args.output = os.getcwd() + os.sep + creator.ref_pkg.prefix + "_treesapp_refpkg" + os.sep
+
+    if len(creator.ref_pkg.prefix) > 10:
+        logging.error("Name should be <= 10 characters.\n")
+        sys.exit(13)
+
+    # TODO: Check the substitution model for compatibility with RAxML-NG
+
+    if args.cluster:
+        if args.multiple_alignment:
+            logging.error("--cluster and --multiple_alignment are mutually exclusive!\n")
+            sys.exit(13)
+        if not 0.5 <= float(args.similarity) <= 1.0:
+            if 0.5 < float(args.similarity) / 100 < 1.0:
+                args.similarity = str(float(args.similarity) / 100)
+                logging.warning("--similarity  set to {} for compatibility with VSEARCH.\n".format(args.similarity))
+            else:
+                logging.error("--similarity {} is not between the supported range [0.5-1.0].\n".format(args.similarity))
+                sys.exit(13)
+
+    if args.taxa_lca and not args.cluster:
+        logging.error("Unable to perform LCA for representatives without clustering information: " +
+                      "either with a provided VSEARCH file or by clustering within the pipeline.\n")
+        sys.exit(13)
+
+    if args.guarantee and not args.cluster:
+        logging.error("--guarantee used but without clustering there is no reason for it.\n" +
+                      "Include all sequences in " + args.guarantee +
+                      " in " + creator.input_sequences + " and re-run without --guarantee\n")
+        sys.exit(13)
+
+    if args.profile:
+        if not os.path.isfile(args.profile):
+            logging.error("Unable to find HMM profile at '" + args.profile + "'.\n")
+            sys.exit(3)
+        creator.hmm_profile = args.profile
+
+    # Names of files and directories to be created
+    creator.phy_dir = os.path.abspath(creator.var_output_dir) + os.sep + "phylogeny_files" + os.sep
+    creator.training_dir = os.path.abspath(creator.var_output_dir) + os.sep + "placement_trainer" + os.sep
+    creator.hmm_purified_seqs = creator.var_output_dir + creator.ref_pkg.prefix + "_hmm_purified.fasta"
+    creator.filtered_fasta = creator.var_output_dir + creator.sample_prefix + "_filtered.fa"
+    creator.cluster_input = creator.var_output_dir + creator.sample_prefix + "_cluster_input.fasta"
+    creator.clusters_prefix = creator.var_output_dir + creator.sample_prefix + "_cluster" + str(creator.ref_pkg.pid)
+    creator.unaln_ref_fasta = creator.var_output_dir + creator.ref_pkg.prefix + "_ref.fa"
+    creator.phylip_file = creator.var_output_dir + creator.ref_pkg.prefix + ".phy"
+
+    # Ensure the phylogenetic tree output directory from a previous run isn't going to be over-written
+    if not os.path.exists(creator.phy_dir):
+        os.mkdir(creator.phy_dir)
+    else:
+        logging.error(creator.phy_dir + " already exists from a previous run! " +
+                      "Please delete or rename it and try again.\n")
+        sys.exit(13)
+
+    if not os.path.isdir(creator.training_dir):
+        os.mkdir(creator.training_dir)
+
+    return
+
+
 def create_new_ref_fasta(out_fasta, ref_seq_dict, dashes=False):
     """
     Writes a new FASTA file using a dictionary of ReferenceSequence class objects
@@ -97,6 +176,7 @@ def present_cluster_rep_options(cluster_dict: dict, refseq_objects: dict, header
     if not important_seqs:
         important_seqs = set()
     candidates = dict()
+    seq_names = list(refseq_objects.keys())
     for cluster_id in sorted(cluster_dict, key=int):
         cluster_info = cluster_dict[cluster_id]
         acc = 1
@@ -112,20 +192,26 @@ def present_cluster_rep_options(cluster_dict: dict, refseq_objects: dict, header
             raise AssertionError("Unable to find " + cluster_info.representative + " in ReferenceSequence objects!")
 
         if len(cluster_info.members) >= 1 and cluster_info.representative not in important_seqs:
+            if len(cluster_info.members) == 1:
+                member_id, perc_id = cluster_info.members[0]
+                if cluster_info.representative == member_id:
+                    continue
             # Find the EntrezRecords corresponding to each member so they can be displayed
             for cluster_member_info in cluster_info.members:
-                for treesapp_id in sorted(refseq_objects, key=int):
-                    formatted_header = header_registry[treesapp_id].original
-                    if formatted_header == cluster_member_info[0]:
+                if cluster_member_info[0] == cluster_info.representative:
+                    continue
+                for treesapp_id in sorted(seq_names, key=int):
+                    if header_registry[treesapp_id].original == cluster_member_info[0]:
                         refseq_objects[treesapp_id].cluster_rep_similarity = cluster_member_info[1]
                         candidates[str(acc)] = refseq_objects[treesapp_id]
                         acc += 1
+                        seq_names.pop(seq_names.index(treesapp_id))
                         break
             sys.stderr.write("Sequences in '" + cluster_info.lca + "' cluster:\n")
             for num in sorted(candidates.keys(), key=int):
                 sys.stderr.write("\t{}. ".format(num) +
                                  '\t'.join(["{} | {}\t".format(candidates[num].organism, candidates[num].accession),
-                                            str(len(candidates[num].sequence)) + "bp or aa",
+                                            str(len(candidates[num].sequence)) + " bp or aa",
                                             str(candidates[num].cluster_rep_similarity)]))
                 if each_lineage:
                     sys.stderr.write("\t" + "(lineage = " + candidates[num].lineage + ")")
@@ -140,7 +226,7 @@ def present_cluster_rep_options(cluster_dict: dict, refseq_objects: dict, header
                 best = input("Invalid number. Number of the best representative? ")
             for num_id in candidates:
                 if num_id != best:
-                    candidates[best].cluster_rep = False
+                    candidates[num_id].cluster_rep = False
 
     return
 
@@ -234,7 +320,7 @@ def strip_rank_prefix_from_organisms(entrez_record_dict: dict, taxa_trie: Taxono
 
 def remove_by_truncated_lineages(fasta_records: dict, min_taxonomic_rank: str, taxa_hierarchy: TaxonomicHierarchy,
                                  guarantees=None) -> dict:
-    if min_taxonomic_rank == 'k':
+    if min_taxonomic_rank == 'r':
         return fasta_records
 
     num_removed = 0
@@ -327,30 +413,36 @@ def summarize_reference_taxa(reference_dict: dict, t_hierarchy: TaxonomicHierarc
     return taxonomic_summary_string
 
 
-def lineages_to_dict(fasta_replace_dict: dict, taxa_lca=False) -> dict:
+def lineages_to_dict(fasta_entrez_records: dict, taxa_lca=False) -> dict:
     """
     Populates the organism, accession ID and lineage information contained in ReferencePackage.lineage_ids
 
-    :param fasta_replace_dict: Dictionary mapping numbers (internal treesapp identifiers) to ReferenceSequence objects
+    :param fasta_entrez_records: Dictionary mapping numbers (internal treesapp identifiers) to ReferenceSequence objects
     :param taxa_lca: Flag indicating whether a cluster's lineage is just the representatives or the LCA of all members
     :return: A dictionary mapping TreeSAPP reference node IDs to a string with their organism, accession and lineage
     """
     no_lineage = list()
     ref_lineage_map = {}
-    for treesapp_id in sorted(fasta_replace_dict.keys(), key=int):  # type: str
+    for treesapp_id in sorted(fasta_entrez_records.keys(), key=int):  # type: str
         # Definitely will not uphold phylogenetic relationships but at least sequences
         # will be in the right neighbourhood rather than ordered by their position in the FASTA file
-        reference_sequence = fasta_replace_dict[treesapp_id]  # type: entrez_utils.EntrezRecord
+        reference_sequence = fasta_entrez_records[treesapp_id]  # type: entrez_utils.EntrezRecord
+
+        if not reference_sequence.versioned:
+            logging.error("'versioned' attribute has not been set for the following EntrezRecord:\n"
+                          "{}\n".format(reference_sequence.get_info()))
+            raise AssertionError("Unable to create the sequence lineage map.")
+
         if taxa_lca:
             lineage = reference_sequence.cluster_lca
         else:
             lineage = reference_sequence.lineage
         if not lineage:
-            no_lineage.append(reference_sequence.accession)
+            no_lineage.append(reference_sequence.versioned)
             lineage = ''
 
         ref_lineage_map[treesapp_id] = "{0} | {1}\t{2}".format(reference_sequence.organism,
-                                                               reference_sequence.accession,
+                                                               reference_sequence.versioned,
                                                                lineage)
 
     if len(no_lineage) > 0:
@@ -360,28 +452,6 @@ def lineages_to_dict(fasta_replace_dict: dict, taxa_lca=False) -> dict:
     return ref_lineage_map
 
 
-def parse_model_parameters(placement_trainer_file):
-    """
-    Returns the model parameters on the line formatted like
-     'Regression parameters = (m,b)'
-    in the file placement_trainer_results.txt
-    :return: tuple
-    """
-    trainer_result_re = re.compile(r"^Regression parameters = \(([0-9,.-]+)\)$")
-    try:
-        trainer_handler = open(placement_trainer_file, 'r')
-    except IOError:
-        logging.error("Unable to open '" + placement_trainer_file + "' for reading!\n")
-        sys.exit(3)
-    params = None
-    for line in trainer_handler:
-        match = trainer_result_re.match(line)
-        if match:
-            params = match.group(1).split(',')
-    trainer_handler.close()
-    return params
-
-
 def remove_outlier_sequences(fasta_record_objects: dict, od_seq_exe: str, mafft_exe: str,
                              output_dir="./outliers", num_threads=2) -> None:
     od_input = output_dir + "od_input.fasta"
@@ -430,7 +500,7 @@ def guarantee_ref_seqs(cluster_dict: dict, important_seqs: set) -> dict:
     nonredundant_guarantee_cluster_dict = dict()  # Will be used to replace cluster_dict
     expanded_cluster_id = 0
     for cluster_id in sorted(cluster_dict, key=int):
-        cluster_inst = cluster_dict[cluster_id]  # type: classy.Cluster
+        cluster_inst = cluster_dict[cluster_id]  # type: seq_clustering.Cluster
         representative = cluster_inst.representative
         if len(cluster_inst.members) == 0:
             if representative in important_seqs:
@@ -443,7 +513,7 @@ def guarantee_ref_seqs(cluster_dict: dict, important_seqs: set) -> dict:
             while x < len(cluster_inst.members):
                 member = cluster_inst.members[x]
                 if member[0] in important_seqs:
-                    nonredundant_guarantee_cluster_dict[expanded_cluster_id] = classy.Cluster(member[0])
+                    nonredundant_guarantee_cluster_dict[expanded_cluster_id] = seq_clustering.Cluster(member[0])
                     nonredundant_guarantee_cluster_dict[expanded_cluster_id].members = []
                     nonredundant_guarantee_cluster_dict[expanded_cluster_id].lca = cluster_inst.lca
                     expanded_cluster_id += 1
@@ -491,7 +561,7 @@ def find_cluster_lca(cluster_dict: dict, fasta_record_objects: dict, header_regi
 
     lineages = list()
     for cluster_id in sorted(cluster_dict, key=int):
-        cluster_inst = cluster_dict[cluster_id]  # type: classy.Cluster
+        cluster_inst = cluster_dict[cluster_id]  # type: seq_clustering.Cluster
         members = [cluster_inst.representative]
         # format of member list is: [header, identity, member_seq_length/representative_seq_length]
         members += [member[0] for member in cluster_inst.members]
@@ -514,7 +584,7 @@ def find_cluster_lca(cluster_dict: dict, fasta_record_objects: dict, header_regi
 
 def formulate_train_command(input_seqs: str, ref_pkg, output_dir: str, args, acc_to_lin=None, seqs_to_lin=None) -> list:
     trainer_cmd = ["-i", input_seqs,
-                   "-r", ref_pkg.f__json,
+                   "-r", ref_pkg.f__pkl,
                    "-o", output_dir,
                    "-m", ref_pkg.molecule,
                    "--num_procs", str(args.num_threads),
