@@ -1109,6 +1109,8 @@ class Lineage:
             # Cut the lineage at the first empty rank
             for rank in self.rank_attributes:
                 taxon = self.__dict__[rank]
+                if taxon is None:
+                    break
                 taxa.append(taxon)
             self.Lineage = self.lin_sep.join(taxa)
 
@@ -1259,21 +1261,29 @@ def map_orf_lineages(seq_lineage_tbl: str, header_registry: dict, refpkg_name=No
 
     pbar = tqdm(total=len(seq_lineage_map), ncols=100)
 
-    for seq_name in sorted(seq_lineage_map):  # type: (str, Lineage)
+    for seq_name in sorted(seq_lineage_map):  # type: str
         # Its slow to perform so many re.search's but without having a guaranteed ORF pattern
         # we can't use hash-based data structures to bring it to O(N)
         parent_re = re.compile(re.escape(seq_name))
         x = 0
         while x < len(treesapp_nums):
             header = header_registry[treesapp_nums[x]]
+            if not header.first_split:
+                logging.error("Sequence header '{}'s first_split attribute hasn't been set.\n".format(header.original))
+                raise AssertionError("Sequence header '{}'s first_split attribute hasn't been set.\n"
+                                     .format(header.original))
+
             assigned_seq_name = re.sub(r"\|{0}\|\d+_\d+.*".format(refpkg_name), '', header.original)
             if parent_re.search(assigned_seq_name):
                 curr_match = parent_re.search(assigned_seq_name)  # type: re.Match
                 if assigned_seq_name not in seq_name:
                     # Prevent seq_lineage_tbl 'seq_name's that are incomplete words from matching header names
-                    if not re.match(string=assigned_seq_name[curr_match.end()], pattern=r"[_\-\.,;: |]"):
-                        x += 1
-                        continue
+                    try:
+                        if not re.match(string=assigned_seq_name[curr_match.end()], pattern=r"[_\-\.,;: |]"):
+                            x += 1
+                            continue
+                    except IndexError:
+                        pass
                 # Now ensure that this is the best+longest match
                 if header.first_split not in classified_seq_lineage_map:
                     classified_seq_lineage_map[header.first_split] = seq_name
