@@ -61,6 +61,12 @@ class PhyloPlace:
 
         return
 
+    def set_attribute_types(self, pquery_dists: str) -> None:
+        self.distal_length, self.pendant_length, self.mean_tip_length = [float(d) for d in pquery_dists.split(',')]
+        self.edge_num = int(self.edge_num)
+        self.like_weight_ratio = float(self.like_weight_ratio)
+        return
+
     def calc_mean_tip_length(self, internal_leaf_node_map: dict, ref_tree, memoization_map=None) -> None:
         if isinstance(ref_tree, str):
             ref_tree = load_ete3_tree(ref_tree)
@@ -169,11 +175,23 @@ class PQuery:
         self.end = 0
         self.seq_len = 0
 
+    def __str__(self):
+        return "PQuery instance for '{}' placed on RefPkg '{}' with {} placements".format(self.place_name,
+                                                                                          self.ref_name,
+                                                                                          len(self.placements))
+
     def clear(self):
         self.node_map.clear()
         self.placements.clear()
         return
 
+    def set_attribute_types(self) -> None:
+        self.end = int(self.end)
+        self.start = int(self.start)
+        self.abundance = float(self.abundance)
+        self.avg_evo_dist = float(self.avg_evo_dist)
+        return
+
     # def transfer_metadata(self, jplace_inst):
     #     self.tree = jplace_inst.tree
     #     self.fields = jplace_inst.fields
@@ -196,6 +214,13 @@ class PQuery:
         summary_str += "\tTaxonomic rank resolved to is {}\n"
         return summary_str
 
+    def string_distances(self) -> None:
+        dist_ar = [self.consensus_placement.distal_length,
+                   self.consensus_placement.pendant_length,
+                   self.consensus_placement.mean_tip_length]
+        self.distances = ','.join([str(round(x, 4)) for x in dist_ar])
+        return
+
     def name_placed_sequence(self) -> None:
         names = set()
         for pplace in self.placements:  # type: PhyloPlace
@@ -245,8 +270,7 @@ class PQuery:
         :param leaf_abundance_sums: A dictionary mapping tree leaf numbers to abundances (RPKM sums)
         :return: A dictionary mapping numerical leaf node identifiers to normalized abundance values
         """
-        jplace_node = self.consensus_placement.edge_num
-        tree_leaves = self.node_map[jplace_node]
+        tree_leaves = self.node_map[self.consensus_placement.edge_num]
         try:
             normalized_abundance = float(self.abundance/len(tree_leaves))
         except TypeError:
@@ -480,13 +504,10 @@ def assignments_to_pqueries(classified_lines: list) -> dict:
                           '\t'.join(fields) + "\n")
             sys.exit(21)
         pquery.place_name = "{}|{}|{}_{}".format(pquery.seq_name, pquery.ref_name, pquery.start, pquery.end)
-        pquery.end = int(pquery.end)
-        pquery.start = int(pquery.start)
-        pquery.abundance = float(pquery.abundance)
+        pquery.set_attribute_types()
         pquery.seq_len = pquery.end - pquery.start
         pquery.lct = pquery.recommended_lineage
-        con_place.distal_length, con_place.pendant_length, con_place.mean_tip_length = [float(d) for d in
-                                                                                        pquery.distances.split(',')]
+        con_place.set_attribute_types(pquery.distances)
         pquery.consensus_placement = con_place
         try:
             pqueries[pquery.ref_name].append(pquery)
@@ -551,6 +572,17 @@ class TreeLeafReference:
         self.accession = ""
         self.complete = False
 
+    def match_tree_leaf(self, leaf_name: str, refpkg_name="") -> bool:
+        if leaf_name == self.description:
+            return True
+        elif leaf_name in self.description.split(" | "):
+            return True
+        elif leaf_name == self.number:
+            return True
+        elif refpkg_name and leaf_name == self.number + "_" + refpkg_name:
+            return True
+        return False
+
     def summarize_tree_leaf(self):
         summary_string = "Leaf ID:\n\t{}\n".format(str(self.number)) +\
                          "Description:\n\t'{}'\n".format(str(self.description))
