@@ -8,7 +8,7 @@ import os
 import logging
 
 from collections import namedtuple
-from seaborn import color_palette
+import seaborn
 
 from treesapp.phylo_seq import TreeLeafReference
 from treesapp.classy import TreeSAPP
@@ -30,13 +30,11 @@ class PhyPainter(TreeSAPP):
         super(PhyPainter, self).__init__("colour")
         self.refpkg_dict = {}
         self.refpkg_leaf_nodes_to_colour = {}
-        self.taxa_phenotype_map = {}
         self.taxa_to_colour = set()
-        self.phenotypes = False
         self.rank = ""
         self.palette = ""
         self.set_op = ""
-        self.output_prefix = ""
+        self.feature_name = ""
         self.num_taxa = 0
         self.num_seqs = 0
         self.rank_depth = -1
@@ -46,7 +44,7 @@ class PhyPainter(TreeSAPP):
         self.ref_pkg = None
         for pkl_file in args.pkg_path:
             ref_pkg = ReferencePackage()
-            ref_pkg.f__json = pkl_file
+            ref_pkg.f__pkl = pkl_file
             ref_pkg.slurp()
             if ref_pkg.validate():
                 self.refpkg_dict[ref_pkg.prefix] = ref_pkg
@@ -74,15 +72,12 @@ class PhyPainter(TreeSAPP):
             self.rank = args.rank
         self.palette = args.palette
 
-        if args.phenotypes:
-            self.phenotypes = True
-
-        if args.name:
-            self.output_prefix = args.name
-        elif self.phenotypes:
-            self.output_prefix, _ = os.path.splitext(os.path.basename(args.phenotypes))
+        # Determine the prefix for the colour files
+        if args.attribute == "taxonomy":
+            self.feature_name = self.rank
         else:
-            self.output_prefix = self.rank
+            self.feature_name = args.attribute
+            self.rank = None
 
         return
 
@@ -259,6 +254,27 @@ class PhyPainter(TreeSAPP):
 
         return
 
+    def get_colours(self) -> list:
+        n_clade = len(self.taxa_to_colour)
+        if n_clade <= 1 and self.rank:
+            logging.warning("{} unique clade(s) identified at rank '{}'."
+                            " Consider using a more resolved rank or adjusting other filtering parameters.\n"
+                            "".format(n_clade, self.feature_name))
+        else:
+            logging.debug("Identified {} unique clades for '{}'.\n".format(n_clade, self.feature_name))
+
+        try:
+            colours = seaborn.color_palette(self.palette, n_clade).as_hex()
+            logging.info("Using palette '{}' for styling.\n".format(self.palette))
+        except ValueError as e:
+            logging.error(str(e) + "\n")
+            sys.exit(4)
+
+        if len(colours) != n_clade:
+            logging.error("Bad colour parsing! len(colours) != number of clades.\n")
+            raise ValueError
+        return colours
+
 
 def convert_clades_to_ranges(taxa_clades: dict, leaf_order: list) -> dict:
     taxa_ranges = dict()
@@ -287,29 +303,6 @@ def create_write_file(file_name: str, text: str) -> None:
     return
 
 
-def get_colours(clade_names: set, palette: str, rank: str) -> list:
-    if len(clade_names) <= 1:
-        logging.error("{} unique clade(s) identified at rank '{}'."
-                      " Consider changing rank to something more specific or adjusting other filtering paramters.\n"
-                      "".format(len(clade_names), rank))
-        sys.exit(4)
-    else:
-        logging.debug("Identified {} unique clades at {} rank.\n".format(len(clade_names), rank))
-
-    n_clade = len(clade_names)
-    try:
-        colours = color_palette(palette, n_clade).as_hex()
-        logging.info("Using palette '{}' for styling.\n".format(palette))
-    except ValueError as e:
-        logging.error(str(e) + "\n")
-        sys.exit(4)
-
-    if len(colours) != n_clade:
-        logging.error("Bad colour parsing! len(colours) != number of clades.\n")
-        sys.exit(8)
-    return colours
-
-
 def map_colours_to_taxa(taxa_order, colours):
     """
     Function for mapping
@@ -412,55 +405,8 @@ def order_taxa(taxa_to_colour: set, taxon_leaf_map: dict, leaf_order: list):
     return taxon_order
 
 
-def read_phenotypes(phenotypes_file: str, comment_char='#') -> dict:
-    taxa_phenotype_map = {}
-    try:
-        file_handler = open(phenotypes_file, 'r')
-    except IOError:
-        logging.error("Unable to open taxa-phenotype table '{}' for reading.\n".format(phenotypes_file))
-        sys.exit(7)
-
-    for line in file_handler:
-        if not line or line[0] == comment_char:
-            continue
-        if line.find(comment_char) >= 0:
-            line = line[:line.find(comment_char)]
-        try:
-            taxon_name, phenotype = line.rstrip().split("\t")
-        except ValueError:
-            logging.error("Unable to parse line in {}:\n{}\n".format(phenotypes_file, line))
-            sys.exit(9)
-        if taxon_name in taxa_phenotype_map:
-            logging.warning("Taxon '{}' found in {} multiple times and will be overwritten.\n"
-                            "".format(taxon_name, phenotypes_file))
-        taxa_phenotype_map[taxon_name.strip()] = phenotype.strip()
-
-    file_handler.close()
-
-    return taxa_phenotype_map
-
-
-def map_taxa_to_leaf_nodes(leaf_names: list, refpkg: ReferencePackage) -> dict:
-    taxon_leaf_map = {}
-    i = 0
-    # Collect the leaf names for taxa names
-    while i < len(leaf_names):
-        taxon_name = leaf_names[i]  # type: str
-        if taxon_name not in refpkg.taxa_trie.hierarchy:
-            i += 1
-        else:
-            taxon_leaf_nodes = refpkg.get_leaf_nodes_for_taxon(taxon_name)
-            taxon_leaf_map[taxon_name] = [leaf.number + '_' + refpkg.prefix for leaf in taxon_leaf_nodes]
-            leaf_names.pop(i)
-    # The remaining names in leaf_names refer to individual leaf nodes, not higher-level taxa
-    for leaf_name in leaf_names:
-        leaves = refpkg.get_leaf_node_by_name(leaf_name)
-        if leaves:
-            taxon_leaf_map[leaf_name] = [leaf.number + '_' + refpkg.prefix for leaf in leaves]
-    return taxon_leaf_map
-
-
 def convert_taxa_to_phenotypes(taxon_leaf_map: dict, phenotypes_map: dict) -> dict:
+    """Creates a dictionary mapping feature annotation names to a list of leaf node names."""
     phenotype_leaf_map = {}
     while taxon_leaf_map:
         taxon, leaves = taxon_leaf_map.popitem()  # type: (str, list)
