@@ -2,20 +2,24 @@ import logging
 import os
 import re
 import sys
-import json
 import inspect
+from glob import glob
 from shutil import copy
+from datetime import datetime as dt
 
 from packaging import version
 from ete3 import Tree
+from pandas import DataFrame
 import joblib
 
 from treesapp.phylo_seq import TreeLeafReference
-from treesapp.entish import annotate_partition_tree, label_internal_nodes_ete, verify_bifurcations
+from treesapp import entish
 from treesapp.external_command_interface import launch_write_command
 from treesapp.fasta import read_fasta_to_dict, write_new_fasta, multiple_alignment_dimensions, FASTA, register_headers
 from treesapp.taxonomic_hierarchy import TaxonomicHierarchy, Taxon
 from treesapp.utilities import base_file_prefix, load_taxonomic_trie, match_file, get_hmm_value
+from treesapp import file_parsers
+from treesapp.clade_annotation import CladeAnnotation
 from treesapp import wrapper
 from treesapp import __version__ as ts_version
 
@@ -30,12 +34,12 @@ class ReferencePackage:
     """
     def __init__(self, refpkg_name=""):
         self.prefix = refpkg_name
-        self.refpkg_code = "Z1111"  # AKA denominator
+        self.date = dt.now().strftime("%Y-%m-%d")  # Date the reference package was created
+        self.refpkg_code = refpkg_name + dt.now().strftime("%m%d.%s")  # AKA denominator
 
         # These are files (with '_f' suffix) and their respective data (read with file.readlines())
-        # TODO: Rename to f__pkl
         self.refpkg_suffix = "_build.pkl"
-        self.f__json = self.prefix + self.refpkg_suffix  # Path to the pickled reference package file
+        self.f__pkl = self.prefix + self.refpkg_suffix  # Path to the pickled reference package file
         self.msa = []  # Reference MSA FASTA
         self.f__msa = self.prefix + ".fa"
         self.profile = []
@@ -49,12 +53,12 @@ class ReferencePackage:
         self.model_info = []
         self.f__model_info = self.prefix + "_epa.model"  # RAxML-NG --evaluate model file
         self.svc = None
+        self.training_df = DataFrame()
         self.lineage_ids = dict()  # Reference sequence lineage map
 
         # These are metadata values
         self.ts_version = ts_version
         self.sub_model = ""  # EPA-NG compatible substitution model
-        self.date = ""  # Date the reference package was created
         self.update = ""  # Date the reference package was last updated
         self.num_seqs = 0  # Number of reference sequences in the MSA, phylogeny
         self.profile_length = 0  # LENG of the profile HMM (not search profile)
@@ -63,6 +67,7 @@ class ReferencePackage:
         self.tree_tool = ""  # Software used for inferring the reference phylogeny
         self.description = ""
         self.pid = 1.0  # Proportional sequence similarity inputs were clustered at
+        self.feature_annotations = {}  # A dictionary storing the annotations for clades
         self.pfit = []  # Parameters for the polynomial regression function
         self.cmd = ""  # The command used for building the reference package
 
@@ -80,6 +85,11 @@ class ReferencePackage:
         for attr, value in self.__dict__.items():
             yield attr, value
 
+    def __str__(self):
+        return "TreeSAPP ({}) ReferencePackage '{}' with {} sequences".format(self.ts_version,
+                                                                              self.prefix,
+                                                                              self.num_seqs)
+
     def get_public_attributes(self) -> list:
         return [attr[0] for attr in inspect.getmembers(self) if
                 type(attr[1]) in [int, float, str, list, dict] and
@@ -108,33 +118,37 @@ class ReferencePackage:
 
     def clone(self, clone_path: str):
         refpkg_clone = ReferencePackage()
-        if not os.path.isfile(self.f__json):
+        if not os.path.isfile(self.f__pkl):
             self.pickle_package()
-        refpkg_clone.f__json = self.f__json
+        refpkg_clone.f__pkl = self.f__pkl
         refpkg_clone.slurp()
-        refpkg_clone.f__json = clone_path
+        refpkg_clone.f__pkl = clone_path
         return refpkg_clone
 
-    def pickle_package(self) -> None:
+    def pickle_package(self, new_output_dir=None) -> None:
         """
-        Dumps a new joblib pickled file of the ReferencePackage instance dictionary to self.f__json.
+        Dumps a new joblib pickled file of the ReferencePackage instance dictionary to self.f__pkl.
 
+        :param new_output_dir: Will copy the ReferencePackage pkl to this new directory (its form of 'cp').
         :return: None.
         """
-        if len(self.f__json) == 0:
-            self.bail("ReferencePackage.f__json not set. ReferencePackage band() cannot be completed.\n")
+        if len(self.f__pkl) == 0:
+            self.bail("ReferencePackage.f__pkl not set. ReferencePackage band() cannot be completed.\n")
             raise AttributeError
 
-        output_dir = os.path.dirname(self.f__json)
+        # Ensure the pickled RefPkg can be written to a valid directory
+        if new_output_dir:
+            self.f__pkl = new_output_dir + os.sep + os.path.basename(self.f__pkl)
+        output_dir = os.path.dirname(self.f__pkl)
         if not output_dir:
             output_dir = "./"
         if not os.path.isdir(output_dir):
             os.mkdir(output_dir)
 
         try:
-            refpkg_handler = open(self.f__json, 'wb')
+            refpkg_handler = open(self.f__pkl, 'wb')
         except IOError:
-            self.bail("Unable to open reference package pickled file '{}' for writing.\n".format(self.f__json))
+            self.bail("Unable to open reference package pickled file '{}' for writing.\n".format(self.f__pkl))
             raise IOError
 
         refpkg_dict = {}
@@ -229,7 +243,7 @@ class ReferencePackage:
                           "TreeSAPP will only create a single directory at a time.\n".format(output_dir, self.prefix))
             sys.exit(3)
 
-        output_prefix = os.path.join(output_dir, '_'.join([self.prefix, self.refpkg_code, self.date])) + os.sep
+        output_prefix = os.path.join(output_dir, self.refpkg_code) + os.sep
 
         if not os.path.isdir(output_prefix):
             os.mkdir(output_prefix)
@@ -253,44 +267,40 @@ class ReferencePackage:
 
         :return: None
         """
-        new_path = self.f__json
-        if len(self.f__json) == 0:
-            logging.error("ReferencePackage.f__json was not set.\n")
+        new_path = self.f__pkl
+        if len(self.f__pkl) == 0:
+            logging.error("ReferencePackage.f__pkl was not set.\n")
             sys.exit(11)
 
-        if not os.path.isfile(self.f__json):
-            logging.error("ReferencePackage pickle file '{}' doesn't exist.\n".format(self.f__json))
+        if not os.path.isfile(self.f__pkl):
+            logging.error("ReferencePackage pickle file '{}' doesn't exist.\n".format(self.f__pkl))
             sys.exit(7)
 
         try:
-            refpkg_data = joblib.load(self.f__json)
-        except KeyError:
-            refpkg_handler = open(self.f__json, 'r')
-            refpkg_data = json.load(refpkg_handler)
-            refpkg_handler.close()
+            refpkg_data = joblib.load(self.f__pkl)
         except EOFError:
-            logging.error("Joblib was unable to load reference package pickle '{}'.\n".format(self.f__json))
+            logging.error("Joblib was unable to load reference package pickle '{}'.\n".format(self.f__pkl))
             sys.exit(17)
 
         for a, v in refpkg_data.items():
             self.__dict__[a] = v
 
         # Fix the pickle path
-        self.f__json = new_path
+        self.f__pkl = new_path
 
         if type(self.tree) is list:
             if len(self.tree) > 0:
                 self.tree = self.tree[0]
             else:
                 logging.error("Unable to load the phylogenetic tree for reference package '{}' ({})\n."
-                              "".format(self.prefix, self.f__json))
+                              "".format(self.prefix, self.f__pkl))
                 return
 
         self.load_taxonomic_hierarchy()
 
         return
 
-    def validate(self, check_files=False):
+    def validate(self, check_files=False) -> bool:
         """
         Function that ensures the number of sequences is equal across all files and
         the version of TreeSAPP used to create this reference package is compatible with the current version.
@@ -338,7 +348,7 @@ class ReferencePackage:
                                                                                       self.num_seqs,
                                                                                       len(rt)))
             return False
-        self.tree = verify_bifurcations(self.tree)
+        self.tree = entish.verify_bifurcations(self.tree)
 
         # Compare the number of sequences in lineage IDs
         n_leaf_nodes = len(self.generate_tree_leaf_references_from_refpkg())
@@ -396,7 +406,7 @@ class ReferencePackage:
                 self.tree = self.tree.pop(0)
 
         rt = Tree(self.tree)
-        label_internal_nodes_ete(rt)
+        entish.label_internal_nodes_ete(rt)
         return rt
 
     def get_internal_node_leaf_map(self) -> dict:
@@ -441,15 +451,39 @@ class ReferencePackage:
         :return: List of TreeLeafReference instances
         """
         ref_leaf_nodes = []
-        for treesapp_id in sorted(self.lineage_ids, key=int):
-            seq_name, lineage = self.lineage_ids[treesapp_id].split("\t")
-            ref = TreeLeafReference(treesapp_id, seq_name)
+        for leaf_id in sorted(self.lineage_ids, key=int):
+            seq_name, lineage = self.lineage_ids[leaf_id].split("\t")
+            ref = TreeLeafReference(leaf_id, seq_name)
             if self.taxa_trie.root_taxon != lineage.split(self.taxa_trie.lin_sep)[0]:
                 lineage = self.taxa_trie.root_taxon + self.taxa_trie.lin_sep + lineage
             ref.lineage = lineage
+            try:
+                ref.accession = ref.description.split(' | ')[1]
+            except IndexError:
+                pass
             ref_leaf_nodes.append(ref)
         return ref_leaf_nodes
 
+    def update_lineage_ids(self, new_lineage_map) -> None:
+        """
+        Replaces all lineages in self.lineages with their corresponding values in new_lineage_map.
+
+        :param new_lineage_map: A dictionary mapping taxonomic lineages in the reference package to replacement lineages
+        :return: None
+        """
+        same = []
+        for leaf_id in sorted(self.lineage_ids, key=int):
+            seq_name, lineage = self.lineage_ids[leaf_id].split("\t")
+            if lineage in new_lineage_map:
+                self.lineage_ids[leaf_id] = seq_name + "\t" + new_lineage_map[lineage]
+            else:
+                same.append(leaf_id)
+        if len(same) > 0:
+            logging.info("{} reference sequence lineages were not updated. More info in log.\n".format(len(same)))
+            logging.debug("Reference sequences that remain unchanged:\n\t{}\n"
+                          "".format("\n\t".join(name + "\t" + self.lineage_ids[name] for name in same)))
+        return
+
     def load_taxonomic_hierarchy(self) -> None:
         """
         Loads the ReferencePackage's taxonomic lineages into it's TaxonomicHierarchy (self.taxa_trie) using
@@ -462,6 +496,15 @@ class ReferencePackage:
         self.taxa_trie.validate_rank_prefixes()
         return
 
+    def create_viewable_newick(self) -> str:
+        """Takes the reference package's tree and replaces the node IDs with proper organism name and accessions."""
+        ete_tree = self.get_ete_tree()
+        ref_leaf_index = {ref_leaf.number + '_' + self.prefix: ref_leaf for ref_leaf
+                          in self.generate_tree_leaf_references_from_refpkg()}
+        for leaf_node in ete_tree.get_leaves():
+            leaf_node.name = ref_leaf_index[leaf_node.name].description
+        return ete_tree.get_tree_root().write(features="name")
+
     def create_itol_labels(self, output_dir) -> None:
         """
         Create the marker_labels.txt file for each marker gene that was used for classification
@@ -501,20 +544,6 @@ class ReferencePackage:
     def alignment_dims(self):
         return multiple_alignment_dimensions(self.f__msa)
 
-    def screen_refs_by_lineage(self, target_lineage: str) -> list:
-        """
-        Returns the TreeLeafReference instances from the reference package that match or are children of target_lineage
-
-        :param target_lineage: A string representing the lineage to fish for reference leaf nodes (e.g. d__Archaea)
-        :return: A list of all TreeLeafReference instances that belong to the target_lineage
-        """
-        children = []
-        lineage_re = re.compile(target_lineage)
-        for leaf_node in self.generate_tree_leaf_references_from_refpkg():  # type: TreeLeafReference
-            if lineage_re.search(leaf_node.lineage):
-                children.append(leaf_node)
-        return children
-
     def get_hierarchy_taxon_for_leaf(self, leaf: TreeLeafReference):
         if not leaf.lineage:
             return None
@@ -537,20 +566,71 @@ class ReferencePackage:
                 taxon_leaf_nodes.append(leaf)
         return taxon_leaf_nodes
 
-    def get_leaf_node_by_name(self, leaf_name: str) -> list:
-        leaves = []
-        for leaf_node in self.generate_tree_leaf_references_from_refpkg():  # type: TreeLeafReference
-            if leaf_name == leaf_node.description:
-                leaves.append(leaf_node)
-            desc, acc = leaf_node.description.split(" | ")
-            if leaf_name == desc or leaf_name == acc:
-                leaves.append(leaf_node)
+    def map_taxa_to_leaf_nodes(self, leaf_names: list) -> dict:
+        """
+        A flexible method for mapping a set of nodes by their accessions or a taxon.
 
-        if len(leaves) == 0:
-            logging.warning("Unable to find leaf '{}' in {} reference package leaves.\n".format(leaf_name, self.prefix))
+        :return: A dictionary mapping taxa or leaf node names to leaf node numbers. Example:
+        {'g__Methanosarcina': [53_McrA, 54_McrA], 'WP_1245.1': [62_McrA]}"""
+        taxon_leaf_map = {}
+        i = 0
+        # Collect the leaf names for taxa names
+        while i < len(leaf_names):
+            taxon_name = leaf_names[i]  # type: str
+            if taxon_name not in self.taxa_trie.hierarchy:
+                i += 1
+            else:
+                taxon_leaf_nodes = self.get_leaf_nodes_for_taxon(taxon_name)
+                taxon_leaf_map[taxon_name] = [leaf.number + '_' + self.prefix for leaf in taxon_leaf_nodes]
+                leaf_names.pop(i)
+
+        # The remaining names in leaf_names refer to individual leaf nodes, not higher-level taxa
+        matched_leaves = self.get_leaf_nodes_by_name(leaf_names)
+        mia_leaves = []
+        for ref_leaf in leaf_names:  # type: str
+            try:
+                taxon_leaf_map[ref_leaf] = [leaf.number + '_' + self.prefix for leaf in matched_leaves[ref_leaf]]
+            except KeyError:
+                mia_leaves.append(ref_leaf)
+
+        if len(mia_leaves) > 0:
+            logging.warning("Unable to find all leaves '{}' in {} reference package leaves.\n"
+                            "".format(", ".join(mia_leaves), self.prefix))
+        return taxon_leaf_map
+
+    def get_leaf_nodes_by_name(self, leaf_names) -> dict:
+        leaves = {}
+        leaf_nodes = self.generate_tree_leaf_references_from_refpkg()
+        for leaf_name in leaf_names:  # type: str
+            for tree_leaf in leaf_nodes:  # type: TreeLeafReference
+                if tree_leaf.match_tree_leaf(leaf_name, refpkg_name=self.prefix):
+                    try:
+                        leaves[leaf_name].append(tree_leaf)
+                    except KeyError:
+                        leaves[leaf_name] = [tree_leaf]
 
         return leaves
 
+    def match_taxon_to_internal_nodes(self, taxon_name: str) -> list:
+        """Returns the minimal set of internal nodes that cover all leaf nodes belonging to taxon_name."""
+        internal_nodes = []
+        try:
+            taxon_name, leaf_node_names = self.map_taxa_to_leaf_nodes([taxon_name]).popitem()
+        except KeyError:
+            logging.warning("Unable to match taxon '{}' to internal nodes as taxon isn't in reference package.\n"
+                            "".format(taxon_name))
+            return internal_nodes
+
+        leaf_node_names = set(leaf_node_names)
+        inode_leaf_map = self.get_internal_node_leaf_map()
+        for i_node in reversed(sorted(inode_leaf_map, key=lambda x: len(inode_leaf_map[x]))):
+            if not leaf_node_names:
+                break
+            if leaf_node_names.issuperset(inode_leaf_map[i_node]) or leaf_node_names == inode_leaf_map[i_node]:
+                internal_nodes.append(i_node)
+                leaf_node_names = leaf_node_names.difference(inode_leaf_map[i_node])
+        return internal_nodes
+
     def map_rank_representatives_to_leaves(self, rank_name: str) -> (dict, dict):
         """
         Loads unique taxa found in the tree into a dictionary where the values are all leaves of a taxon.
@@ -742,7 +822,7 @@ class ReferencePackage:
     def recover_raxmlng_supports(self, phylogeny_dir: str) -> None:
         # Annotate the bootstrapped phylogeny
         bootstrap_tree = match_file(phylogeny_dir + "*.raxml.support")
-        annotate_partition_tree(self.prefix, self.generate_tree_leaf_references_from_refpkg(), bootstrap_tree)
+        entish.annotate_partition_tree(self.prefix, self.generate_tree_leaf_references_from_refpkg(), bootstrap_tree)
         copy(bootstrap_tree, self.f__boot_tree)
         return
 
@@ -909,15 +989,6 @@ class ReferencePackage:
                       (len(lineage_reps), dereplication_rank))
         return
 
-    def load_pfit_params(self, build_param_line):
-        """
-        Loads the polynomial classifier parameters that are stored in the ReferencePackage.pfit attribute
-        """
-        build_param_fields = build_param_line.split("\t")
-        if build_param_fields[8]:
-            self.pfit = [float(x) for x in build_param_fields[8].split(',')]
-        return
-
     def enumerate_taxonomic_lineages(self) -> dict:
         """
         At times, it is handy to know how many of each taxonomic label are present in the reference package.
@@ -1007,15 +1078,121 @@ class ReferencePackage:
 
         return rt
 
+    def convert_feature_indices_to_inodes(self, feature_map: dict) -> dict:
+        """Capable of sorting taxa from lineages, internal nodes, leaf names, and descriptions organism names"""
+        internal_node_feature_map = {}
+        unmapped = []
 
-def write_edited_pkl(refpkg: ReferencePackage, output_dir: str, overwrite: bool) -> None:
-    if not overwrite:
-        refpkg.f__json = os.path.join(output_dir, os.path.basename(refpkg.f__json))
-        if os.path.isfile(refpkg.f__json):
-            logging.warning("RefPkg file '{}' already exists.\n".format(refpkg.f__json))
+        # Generate the sets for taxa and internal nodes to compare to
+        taxa_names = self.taxa_trie.get_taxon_names(with_prefix=True)
+        internal_nodes = {str(k): v for k, v in self.get_internal_node_leaf_map().items()}
+
+        for index, feature in feature_map.items():  # type: (str, str)
+            if index in taxa_names:
+                for i_node in self.match_taxon_to_internal_nodes(index):
+                    internal_node_feature_map[i_node] = (feature, index)
+            elif index in internal_nodes:
+                internal_node_feature_map[int(index)] = (feature, '')
+            else:
+                unmapped.append(index)
+
+        taxon_leaf_map = self.map_taxa_to_leaf_nodes(unmapped)
+        for index, feature in feature_map.items():  # type: (str, str)
+            if index not in unmapped:
+                continue
+            try:
+                for i_node in entish.match_leaves_to_internal_nodes(leaf_names=taxon_leaf_map[index],
+                                                                    internal_node_leaf_map=internal_nodes):
+                    internal_node_feature_map[int(i_node)] = (feature, '')
+                unmapped.remove(index)
+            except KeyError:
+                pass
+
+        if unmapped:
+            logging.debug("Unable to find the following feature indices in taxa, leaves or internal nodes:\n\t" +
+                          "\n\t".join(unmapped) + "\n")
+
+        return internal_node_feature_map
+
+    def add_feature_annotations(self, feature_name: str, feature_map: dict, reset=False) -> None:
+        """Triages the different formats of feature maps to their respective functions."""
+        # Sort labels by taxa or internal nodes
+        inode_features = self.convert_feature_indices_to_inodes(feature_map)
+
+        inode_leaf_map = self.get_internal_node_leaf_map()
+        all_leaves = set(sum(inode_leaf_map.values(), []))
+        # Pull the current annotations in case it can be appended to
+        try:
+            clade_annots = self.feature_annotations[feature_name]
+        except KeyError:
+            clade_annots = []
+            self.feature_annotations[feature_name] = clade_annots
+        if reset:
+            self.feature_annotations[feature_name].clear()
+            clade_annots = []
+
+        # Invert the dictionary to map annotation names to taxa
+        feature_map = {}
+        for index, annotation in inode_features.items():  # type: (int, tuple)
+            try:
+                feature_map[annotation].append(index)
+            except KeyError:
+                feature_map[annotation] = [index]
+
+        # Create a new CladeAnnotation instance for each feature
+        for feat_tup, indices in feature_map.items():  # type: (tuple, list)
+            annotation, taxon = feat_tup  # type: (str, str)
+            # Determine whether the feature is already in clade_annots and if so, update it
+            ca = None
+            for prev_annot in clade_annots:  # type: CladeAnnotation
+                if prev_annot.name == annotation:
+                    ca = prev_annot
+                    break
+            if ca is None:
+                ca = CladeAnnotation(name=annotation, key=feature_name)
+                self.feature_annotations[feature_name].append(ca)
+
+            # Populate the taxa and members attributes
+            for index in indices:
+                ca.members.update(inode_leaf_map[index])
+            if taxon:
+                ca.taxa.update([taxon])
+
+        # Remove the feature from feature_annotations it there are no CladeAnnotation entries
+        if len(self.feature_annotations[feature_name]) == 0:
+            self.feature_annotations.pop(feature_name)
+            logging.warning("No clade annotations were made for feature '{}'.\n".format(feature_name))
             return
-    refpkg.pickle_package()
-    return
+
+        # Identify the reference package leaves that are not annotated
+        for ca in self.feature_annotations[feature_name]:
+            for leaf_name in ca.members:
+                try:
+                    all_leaves.remove(leaf_name)
+                except KeyError:
+                    pass  # Probably already present in one or more CladeAnnotation.members
+
+        # Report the leaves that were not annotated by their descriptions and lineages
+        missing_leaf_map = self.get_leaf_nodes_by_name(all_leaves)
+        missing_ref_leaves = sum(missing_leaf_map.values(), [])
+        unique_lineages = sorted(set([ref_leaf.lineage for ref_leaf in missing_ref_leaves]))
+        unique_descs = sorted([ref_leaf.description for ref_leaf in missing_ref_leaves])
+        logging.info("{} references were not annotated. More info can be found in log.\n".format(len(all_leaves)))
+        logging.debug("Unique lineages remaining unannotated:\n\t{}\n"
+                      "Descriptions of unannotated reference leaves:\n\t{}\n".format("\n\t".join(unique_lineages),
+                                                                                     "\n\t".join(unique_descs)))
+
+        return
+
+
+def write_edited_pkl(ref_pkg: ReferencePackage, output_dir: str, overwrite: bool) -> int:
+    if output_dir:
+        ref_pkg.f__pkl = os.path.join(output_dir, os.path.basename(ref_pkg.f__pkl))
+    if not overwrite and os.path.isfile(ref_pkg.f__pkl):
+        logging.warning("RefPkg file '{}' already exists.\n".format(ref_pkg.f__pkl))
+        return 1
+    ref_pkg.pickle_package()
+    return 0
 
 
 def view(refpkg: ReferencePackage, attributes: list) -> None:
@@ -1029,17 +1206,26 @@ def view(refpkg: ReferencePackage, attributes: list) -> None:
 
     for k, v in view_dict.items():
         logging.info("{}:\t".format(k))
-        if type(v) is list and k not in ["pfit"]:
+        if k == "tree":
+            v = refpkg.create_viewable_newick()
+        if isinstance(v, list) and k not in ["pfit"]:  # various file contents
             v = ''.join(v)
-        if type(v) is dict:
-            v = "\n" + "\n".join([sk + "\t" + sv for sk, sv in v.items()])
+        if isinstance(v, dict):  # feature_annotations, lineage_ids
+            buffer = "\n"
+            if len(v.items()) > 0:
+                for sk, sv in v.items():
+                    if isinstance(sv, list):
+                        buffer += sk + "\t" + "".join([str(e) for e in sv]) + "\n"
+                    else:
+                        buffer += sk + "\t" + str(sv) + "\n"
+            v = buffer
         sys.stdout.write(str(v).strip() + "\n")
 
     # TODO: optionally use ReferencePackage.write_refpkg_component
     return
 
 
-def edit(refpkg: ReferencePackage, attributes: list, output_dir, overwrite: bool) -> None:
+def edit(ref_pkg: ReferencePackage, attributes: list, output_dir: str, **kwargs) -> None:
     if len(attributes) > 2:
         logging.error("`treesapp package edit` only edits a single attribute at a time.\n")
         sys.exit(3)
@@ -1047,44 +1233,61 @@ def edit(refpkg: ReferencePackage, attributes: list, output_dir, overwrite: bool
         logging.error("`treesapp package edit` requires a value to change.\n")
         sys.exit(3)
     else:
-        k, v = attributes  # type: (str, str)
+        k, v = attributes  # type: (str, any)
 
     try:
-        current_v = refpkg.__dict__[k]
+        current_v = ref_pkg.__dict__[k]
     except KeyError:
         logging.error("Attribute '{}' doesn't exist in ReferencePackage.\n".format(k))
         sys.exit(1)
 
-    logging.info("Replacing attribute '{}' (currently '{}')\n".format(k, current_v))
+    logging.info("Updating attribute '{}' (currently '{}')\n".format(k, current_v))
 
     if k.startswith("f__"):  # This is a file that is being modified and the contents need to be read
         k_content = re.sub(r"^f__", '', k)
         if not os.path.isfile(v):
             logging.error("Unable to find path to new file '{}'. "
-                          "Exiting and the reference package '{}' will remain unchanged.\n".format(v, refpkg.f__json))
+                          "Exiting and the reference package '{}' will remain unchanged.\n".format(v, ref_pkg.f__pkl))
             sys.exit(5)
         with open(v) as content_handler:
             v_content = content_handler.readlines()
         try:
-            refpkg.__dict__[k_content] = v_content
+            ref_pkg.__dict__[k_content] = v_content
         except KeyError:
-            logging.error("Unable to find ReferencePackage attribute '{}' in '{}'.\n".format(k_content, refpkg.f__json))
+            logging.error("Unable to find ReferencePackage attribute '{}' in '{}'.\n".format(k_content, ref_pkg.f__pkl))
             sys.exit(7)
-    refpkg.__dict__[k] = v
+    elif k == "feature_annotations":
+        if "phenotypes" not in kwargs or not os.path.isfile(kwargs["phenotypes"]):
+            logging.error("A taxonomy-phenotype table must be provided to update the feature_annotations attribute.\n")
+            sys.exit(7)
+        taxa_phenotype_map = file_parsers.read_phenotypes(kwargs["phenotypes"])
+        ref_pkg.add_feature_annotations(feature_name=v, feature_map=taxa_phenotype_map, reset=kwargs["reset"])
+        v = ref_pkg.feature_annotations
+    elif k == "lineage_ids":
+        if kwargs["join"]:
+            lineage_map = file_parsers.read_lineage_map(v)
+            ref_pkg.update_lineage_ids(lineage_map)
+            v = ref_pkg.lineage_ids
+        else:
+            v = file_parsers.read_lineage_ids(v)
+    ref_pkg.__dict__[k] = v
 
-    write_edited_pkl(refpkg, output_dir, overwrite)
+    if ref_pkg.validate() is False:
+        sys.exit(17)
+
+    write_edited_pkl(ref_pkg, output_dir, kwargs["overwrite"])
 
     return
 
 
-def rename(refpkg: ReferencePackage, attributes: list, output_dir: str, overwrite: bool) -> None:
+def rename(ref_pkg: ReferencePackage, attributes: list, output_dir: str, overwrite: bool) -> None:
     # Ensure that prefix is the first item in attributes
     if attributes.pop(0) != 'prefix':
         logging.error("Incorrect format of 'rename' command. Please follow the format provided in help description.\n")
         sys.exit(3)
     # Attributes can only be length of two - first item is 'prefix' and second is the new name
     if len(attributes) > 1:
-        logging.error("Only one other argument should be provided to treesapp package rename - the new refpkg name.\n")
+        logging.error("Only one other argument should be provided to treesapp package rename - the new ref_pkg name.\n")
         sys.exit(3)
 
     new_prefix = attributes.pop()
@@ -1095,18 +1298,87 @@ def rename(refpkg: ReferencePackage, attributes: list, output_dir: str, overwrit
 
     # Rename all instances of the old prefix
     for attr_name in edit_str_attrs:
-        refpkg.__dict__[attr_name] = re.sub(refpkg.prefix, new_prefix, refpkg.__dict__[attr_name])
+        ref_pkg.__dict__[attr_name] = re.sub(ref_pkg.prefix, new_prefix, ref_pkg.__dict__[attr_name])
 
     for attr_name in edit_iter_attrs:
         i = 0
-        while i < len(refpkg.__dict__[attr_name]):
-            refpkg.__dict__[attr_name][i] = re.sub(refpkg.prefix, new_prefix, refpkg.__dict__[attr_name][i])
+        while i < len(ref_pkg.__dict__[attr_name]):
+            ref_pkg.__dict__[attr_name][i] = re.sub(ref_pkg.prefix, new_prefix, ref_pkg.__dict__[attr_name][i])
             i += 1
 
     # Finally change the value of the prefix once all other attributes have been modified
-    refpkg.prefix = new_prefix
-    refpkg.f__json = refpkg.prefix + refpkg.refpkg_suffix
+    ref_pkg.prefix = new_prefix
+    ref_pkg.f__pkl = ref_pkg.prefix + ref_pkg.refpkg_suffix
 
-    write_edited_pkl(refpkg, output_dir, overwrite)
+    write_edited_pkl(ref_pkg, output_dir, overwrite)
 
     return
+
+
+def load_refpkgs_from_assign_output(assign_intermediates_dir: str) -> dict:
+    """Returns a dictionary of prefix keys to ReferencePackage instance values found in an assign intermediates dir."""
+    refpkg_dict = dict()
+    refpkg_dirs = glob(assign_intermediates_dir + os.sep + "*_RefPkg")
+    if len(refpkg_dirs) == 0:
+        logging.error("Unable to find reference packages written to '{}'.\n".format(assign_intermediates_dir))
+        return refpkg_dict
+
+    for dir_path in refpkg_dirs:
+        rp_name = re.match(r"(.*)_RefPkg", os.path.basename(dir_path)).group(1)
+        ref_pkg = ReferencePackage()
+        ref_pkg.f__pkl = dir_path + os.sep + rp_name + ref_pkg.refpkg_suffix
+        ref_pkg.slurp()
+        refpkg_dict[ref_pkg.prefix] = ref_pkg
+
+    return refpkg_dict
+
+
+def gather_ref_packages(refpkg_data_dir: str, targets=None) -> dict:
+    """
+    Returns a dictionary of ReferencePackage instances for each MarkerBuild instance in the marker_build_dict.
+    Optionally these markers can be subsetted further by a list of targets
+
+    :param refpkg_data_dir: Path to the directory containing TreeSAPP reference packages (JSON-format)
+    :param targets: List of refpkg codes that are desired or an empty list suggesting all refpkgs should be used
+    :return: Dictionary of ReferencePackage.prefix keys indexing their respective instances
+    """
+    refpkg_dict = dict()
+    refpkgs_found = set()
+    logging.debug("Gathering reference package files... ")
+
+    if targets is None:
+        targets = set()
+    else:
+        targets = set(targets)
+
+    refpkg_files = glob(refpkg_data_dir + os.sep + "*_build.pkl")
+    if len(refpkg_files) == 0:
+        logging.error("No reference package files were found in {}.\n".format(refpkg_data_dir))
+        sys.exit(3)
+
+    for rp_file in refpkg_files:
+        ref_pkg = ReferencePackage()
+        ref_pkg.f__pkl = rp_file
+        ref_pkg.slurp()
+        if targets:  # type: list
+            if ref_pkg.prefix not in targets and ref_pkg.refpkg_code not in targets:
+                continue
+            elif ref_pkg.prefix in refpkgs_found and ref_pkg.refpkg_code in refpkgs_found:
+                logging.warning("RefPkg for {} has already been found. Skipping {}.\n".format(ref_pkg.prefix, rp_file))
+            else:
+                match = targets.intersection({ref_pkg.prefix, ref_pkg.refpkg_code}).pop()
+                refpkgs_found.add(match)
+        if ref_pkg.validate():
+            refpkg_dict[ref_pkg.prefix] = ref_pkg
+    logging.debug("done.\n")
+
+    targets = targets.difference(refpkgs_found)
+    if len(targets) > 0:
+        logging.warning("Reference packages for targets {} could not be found.\n".format(", ".join(targets)))
+
+    if len(refpkg_dict) == 0:
+        logging.error("No reference package data was found.\n" +
+                      "Are there reference packages in '{}'?\n".format(refpkg_data_dir))
+        sys.exit(3)
+
+    return refpkg_dict
