@@ -4,9 +4,8 @@ import sys
 import re
 import logging
 from glob import glob
-from datetime import datetime as dt
 
-from treesapp.classy import Evaluator, Creator, PhyTrainer, Updater
+from treesapp.classy import Evaluator, Updater
 from treesapp.utilities import available_cpu_count
 
 
@@ -15,6 +14,7 @@ class TreeSAPPArgumentParser(argparse.ArgumentParser):
     A base argparse ArgumentParser for TreeSAPP with functions to furnish with common arguments.
     This standardizes the interface for a unified aesthetic across all functions: create, classify, evaluate, etc.
     """
+
     def __init__(self, **kwargs):
         """
         Instantiate the argparse argument-parser and create three broad argument groups:
@@ -33,7 +33,8 @@ class TreeSAPPArgumentParser(argparse.ArgumentParser):
         self.fpkm_opts = self.add_argument_group("Abundance options")
         self.io = self.add_argument_group("Inputs and Outputs")
         self.aes = self.add_argument_group("Aesthetic options")
-        self.optopt = self.add_argument_group("Optional options")
+        self.editors = self.add_argument_group("Package edit options")
+        self.optopt = self.add_argument_group("Optional arguments")
         self.taxa_args = self.add_argument_group("Taxonomic-lineage arguments")
         self.svc_opts = self.add_argument_group("Classifier arguments")
         self.miscellany = self.add_argument_group("Miscellaneous options")
@@ -57,7 +58,7 @@ class TreeSAPPArgumentParser(argparse.ArgumentParser):
                                      help='Delete all intermediate files to save disk space.')
 
     def add_input_fastx(self):
-        self.reqs.add_argument('-i', '--fastx_input', required=True, dest="input",
+        self.reqs.add_argument('-i', '--fastx_input', required=True, dest="input", nargs='+',
                                help='An input file containing DNA or protein sequences in either FASTA or FASTQ format')
 
     def add_output_dir(self):
@@ -98,9 +99,9 @@ class TreeSAPPArgumentParser(argparse.ArgumentParser):
                                       " [DEFAULT = False]")
         self.optopt.add_argument('-w', '--min_seq_length', default=30, type=int,
                                  help='minimal sequence length after alignment trimming [DEFAULT = 30]')
-        self.optopt.add_argument('-m', '--molecule', default='dna', choices=['prot', 'dna', 'rrna'],
-                                 help="Type of input sequences "
-                                      "(prot = protein; dna = nucleotide [DEFAULT]; rrna = rRNA)")
+        self.optopt.add_argument('-m', '--molecule', choices=['prot', 'dna', 'rrna'], required=False, default="",
+                                 help="Type of input sequences (prot = protein; dna = nucleotide; rrna = rRNA). "
+                                      "TreeSAPP will guess by default but this may be required if ambiguous.")
 
     def add_search_params(self):
         self.hmmer_args.add_argument("-s", "--stringency",
@@ -112,14 +113,15 @@ class TreeSAPPArgumentParser(argparse.ArgumentParser):
                                           "for it to be considered. [ DEFAULT = 20 ]")
 
     def add_abundance_params(self):
-        self.fpkm_opts.add_argument("--metric", required=False, default="fpkm", choices=["fpkm", "tpm"],
-                                    help="Selects which normalization metric to use, FPKM or TPM.")
+        self.fpkm_opts.add_argument("--metric", required=False, default="tpm", choices=["fpkm", "tpm"],
+                                    help="Selects which normalization metric to use, FPKM or TPM. [ DEFAULT = tpm ]")
         self.fpkm_opts.add_argument("-r", "--reads", required=False, nargs='+',
                                     help="FASTQ file containing to be aligned to predicted genes using BWA MEM")
         self.fpkm_opts.add_argument("-2", "--reverse", required=False, nargs='+', default=[],
                                     help="FASTQ file containing to reverse mate-pair reads to be aligned using BWA MEM")
         self.fpkm_opts.add_argument("-p", "--pairing", required=False, default='pe', choices=['pe', 'se'],
-                                    help="Indicating whether the reads are paired-end (pe) or single-end (se)")
+                                    help="Indicating whether the reads are paired-end (pe) or single-end (se). "
+                                         "[ DEFAULT = pe ]")
 
     def add_phylogeny_params(self):
         self.optopt.add_argument("-b", "--bootstraps",
@@ -138,17 +140,23 @@ class TreeSAPPArgumentParser(argparse.ArgumentParser):
                                  help="Flag to activate outlier detection and removal from multiple sequence alignments"
                                       " using OD-seq. [ DEFAULT = False ]")
 
-    def add_pplace_params(self):
-        self.pplace_args.add_argument("-l", "--min_like_weight_ratio", default=0.1, type=float, dest="min_lwr",
+    def add_pplace_filter_params(self):
+        self.pplace_args.add_argument("--min_like_weight_ratio", default=0.1, type=float, dest="min_lwr",
                                       help="The minimum likelihood weight ratio required for an EPA placement. "
                                            "[ DEFAULT = 0.1 ]")
         self.pplace_args.add_argument("--max_pendant_length", default=2.0, type=float, dest="max_pd",
                                       help="The maximum pendant length distance threshold, "
                                            "beyond which EPA placements are unclassified. [ DEFAULT = 2.0 ]")
+        self.pplace_args.add_argument("--max_evol_distance", default=3.0, type=float, dest="max_evo",
+                                      help="The maximum total evolutionary distance between a query and reference(s), "
+                                           "beyond which EPA placements are unclassified. [ DEFAULT = 3.0 ]")
+
+    def add_pplace_params(self):
+        self.add_pplace_filter_params()
         self.pplace_args.add_argument("--placement_summary", default="max_lwr", choices=["aelw", "max_lwr"],
                                       dest="p_sum",
-                                      help="Controls the algorithm for consolidating multiple phylogenetic placements."
-                                           "Max LWR will take use the phylogenetic placement with greatest LWR."
+                                      help="Controls the algorithm for consolidating multiple phylogenetic placements. "
+                                           "Max LWR will take use the phylogenetic placement with greatest LWR. "
                                            "aELW uses the taxon with greatest accumulated LWR across placements.")
 
     def add_compute_miscellany(self):
@@ -188,11 +196,11 @@ class TreeSAPPArgumentParser(argparse.ArgumentParser):
                                          "[ DEFAULT is no filter ]",
                                     default="", required=False)
         self.taxa_args.add_argument("--min_taxonomic_rank",
-                                    required=False, default='k', choices=['k', 'p', 'c', 'o', 'f', 'g', 's'],
-                                    help="The minimum taxonomic resolution for reference sequences [ DEFAULT = k ].\n")
+                                    required=False, default='r', choices=['r', 'd', 'p', 'c', 'o', 'f', 'g', 's'],
+                                    help="The minimum taxonomic resolution for reference sequences [ DEFAULT = r ].\n")
         self.taxa_args.add_argument("--taxa_lca",
                                     help="Set taxonomy of representative sequences to LCA of cluster member's taxa.\n"
-                                         "[ --cluster or --uc REQUIRED ]",
+                                         "[ --cluster REQUIRED ]",
                                     default=False, required=False, action="store_true")
         self.taxa_args.add_argument("--taxa_norm",
                                     help="[ IN DEVELOPMENT ] Subsample leaves by taxonomic lineage.\n"
@@ -206,15 +214,19 @@ class TreeSAPPArgumentParser(argparse.ArgumentParser):
                                       " [ DEFAULT = class species ]",
                                  choices=["domain", "phylum", "class", "order", "family", "genus", "species"])
 
-    def add_basic_classifier_model_params(self):
-        self.svc_opts.add_argument("--max_examples", required=False, default=1000, type=int,
-                                   help="Limits the number of examples used for training models. [ DEFAULT = 1000 ]")
+    def add_classifier_kernel_param(self):
         self.svc_opts.add_argument("-k", "--svm_kernel", required=False, default="lin",
                                    choices=["lin", "rbf", "poly"], dest="kernel",
                                    help="Specifies the kernel type to be used in the SVM algorithm. "
                                         "It must be either 'lin' 'poly' or 'rbf'. [ DEFAULT = lin ]")
         return
 
+    def add_basic_classifier_model_params(self):
+        self.add_classifier_kernel_param()
+        self.svc_opts.add_argument("--max_examples", required=False, default=1000, type=int,
+                                   help="Limits the number of examples used for training models. [ DEFAULT = 1000 ]")
+        return
+
     def add_advanced_classifier_params(self):
         self.svc_opts.add_argument("--grid_search", default=False, required=False, action="store_true",
                                    help="Perform a grid search across hyperparameters. Binary classifier only.")
@@ -236,10 +248,18 @@ def add_package_arguments(pkg_parser: TreeSAPPArgumentParser, attributes: list):
     pkg_parser.add_refpkg_file_param()
 
     pkg_parser.reqs.add_argument("attributes", nargs="+",
-                                 help="One or more reference package attributes to view. "
+                                 help="One or more reference package attributes to view or edit. "
                                       "Note: edit will only modify a single attribute at a time. "
                                       "Choices include: {}\n".format(', '.join(attributes)))
-    pkg_parser.optopt.add_argument('-o', '--output', default="./", required=False,
+    pkg_parser.editors.add_argument("-t", "--taxa_map", dest="phenotypes", required=False, default=None,
+                                    help="A file mapping unique taxonomic labels to non-unique features "
+                                         "(e.g. activity, pathway, or other phenotype)")
+    pkg_parser.editors.add_argument("--reset", default=False, action="store_true",
+                                    help="Flag to reset the reference package attribute when editing.")
+    pkg_parser.editors.add_argument("--join", default=False, action="store_true",
+                                    help="Flag indicating the input needs to be joined to the existing attribute. "
+                                         "Applicable to: 'lineage_ids'. [ DEFAULT = False ]")
+    pkg_parser.optopt.add_argument('-o', '--output', default=None, required=False,
                                    help='Path to an output directory. '
                                         'Default is the current working directory.')
     return
@@ -248,14 +268,12 @@ def add_package_arguments(pkg_parser: TreeSAPPArgumentParser, attributes: list):
 def add_colour_arguments(colour_parser: TreeSAPPArgumentParser) -> None:
     colour_parser.add_refpkg_file_param()
 
-    colour_parser.io.add_argument("-n", "--name", required=False, default=None,
-                                  help="The prefix name to use when creating iTOL-compatible output files. "
-                                       "By default the taxonomic rank is used or file name of taxa_map if provided.")
+    colour_parser.io.add_argument("-n", "--attribute", required=False, default="taxonomy",
+                                  help="The reference package attribute to colour by. "
+                                       "Either 'taxonomy' or a reference package's layering annotation name. "
+                                       "[ DEFAULT = 'taxonomy' ]")
     colour_parser.io.add_argument("-o", "--output_dir", dest="output", default="./", required=False,
                                   help="Path to the output directory to write the output files. [ DEFAULT = ./ ]")
-    colour_parser.io.add_argument("-t", "--taxa_map", dest="phenotypes", required=False, default=None,
-                                  help="A file mapping unique taxonomic labels to non-unique features "
-                                       "(e.g. activity, pathway, or other phenotype)")
 
     colour_parser.aes.add_argument('-l', "--rank_level", dest="rank", default="order", required=False,
                                    help="The rank to generate unique colours for [ DEFAULT = 'order' ]")
@@ -284,13 +302,6 @@ def add_layer_arguments(parser: TreeSAPPArgumentParser):
     parser.add_refpkg_opt()
     parser.reqs.add_argument("-o", "--treesapp_output", dest="output", required=True,
                              help="The TreeSAPP output directory.")
-    parser.optopt.add_argument("-c", "--colours_style", required=False, nargs='+',
-                               help="The colours_style file exported from iTOL with the annotation information. "
-                                     "To automatically infer the variable name (rather than through `names`). "
-                                     "File name format should be `marker`_`var`.txt. For example: McrA_Metabolism.txt "
-                                     "would create a new column in marker_contig_map.tsv named 'Metabolism'.")
-    parser.optopt.add_argument("-d", "--annot_dir", required=False, default=None,
-                               help="Path to a directory containing iTOL annotation files for layering.")
     return
 
 
@@ -308,11 +319,12 @@ def add_classify_arguments(assign_parser: TreeSAPPArgumentParser) -> None:
     assign_parser.add_search_params()
     assign_parser.add_pplace_params()
     assign_parser.add_compute_miscellany()
+    assign_parser.add_classifier_kernel_param()
     # The required parameters... for which there are currently none. But they would go here!
 
     assign_parser.optopt.add_argument("--svm", default=False, required=False, action="store_true",
                                       help="Uses the support vector machine (SVM) classification filter. "
-                                           "WARNING: Unless you *really* know your refpkg, you probably don't want this.")
+                                           "WARNING: Unless you *really* know your refpkg, you don't want this.")
 
     # The optionals
     assign_parser.optopt.add_argument('-c', '--composition', default="meta", choices=["meta", "single"],
@@ -326,25 +338,27 @@ def add_classify_arguments(assign_parser: TreeSAPPArgumentParser) -> None:
 
     # The miscellany
     assign_parser.miscellany.add_argument('-R', '--reftree', required=False, default="", type=str,
-                                          help="[IN PROGRESS] Reference package that all queries should be immediately and "
-                                               "directly classified as (i.e. homology search step is skipped).")
+                                          help="[IN PROGRESS] Reference package that all queries should be immediately"
+                                               " and directly classified as (i.e. homology search step is skipped).")
     return
 
 
-def add_abundance_arguments(parser: TreeSAPPArgumentParser):
-    parser.add_abundance_params()
-    parser.add_compute_miscellany()
-    parser.add_delete()
-    parser.reqs.add_argument("--treesapp_output", dest="output", required=True,
-                             help="Path to the directory containing TreeSAPP outputs, "
-                                  "including sequences to be used for the update.")
-    parser.optopt.add_argument("--report", choices=["update", "nothing", "append"], required=False, default="append",
-                               help="What should be done with the abundance values? The TreeSAPP classification table "
-                                    "can be overwritten (update), appended or left unchanged. "
-                                    "[ DEFAULT = append ]")
-    parser.optopt.add_argument("--stage", default="continue", required=False,
-                               choices=["continue", "align_map", "sam_sum", "summarise"],
-                               help="The stage(s) for TreeSAPP to execute [DEFAULT = continue]")
+def add_abundance_arguments(abundance_parser: TreeSAPPArgumentParser):
+    abundance_parser.add_abundance_params()
+    abundance_parser.add_refpkg_opt()
+    abundance_parser.add_compute_miscellany()
+    abundance_parser.add_delete()
+    abundance_parser.reqs.add_argument("--treesapp_output", dest="output", required=True,
+                                       help="Path to the directory containing TreeSAPP outputs, "
+                                            "including sequences to be used for the update.")
+    abundance_parser.optopt.add_argument("--report", choices=["update", "nothing", "append"],
+                                         required=False, default="append",
+                                         help="What should be done with the abundance values? "
+                                              "The TreeSAPP classification table can be overwritten (update), "
+                                              "appended or left unchanged. [ DEFAULT = append ]")
+    abundance_parser.optopt.add_argument("--stage", default="continue", required=False,
+                                         choices=["continue", "align_map", "sam_sum", "summarise"],
+                                         help="The stage(s) for TreeSAPP to execute [DEFAULT = continue]")
     return
 
 
@@ -403,90 +417,89 @@ def add_create_arguments(crt_parser: TreeSAPPArgumentParser) -> None:
     return
 
 
-def add_purity_arguments(parser: TreeSAPPArgumentParser) -> None:
+def add_purity_arguments(pur_parser: TreeSAPPArgumentParser) -> None:
     """
     Adds command-line arguments that are specific to *treesapp purity*
 
     :return: None
     """
-    parser.add_io()
-    parser.add_refpkg_file_param()
-    parser.add_seq_params()
-    parser.add_compute_miscellany()
-    parser.optopt.add_argument("-x", "--extra_info", required=False, default=None,
-                               help="File mapping header prefixes to description information.")
-    parser.optopt.add_argument("--stage", default="continue", required=False,
-                               choices=["continue", "lineages", "classify", "calculate"],
-                               help="The stage(s) for TreeSAPP to execute [DEFAULT = continue]")
-    # TODO: Remove --trim_align from command-line options in parser.add_seq_params()
+    pur_parser.add_io()
+    pur_parser.add_refpkg_file_param()
+    pur_parser.add_seq_params()
+    pur_parser.add_compute_miscellany()
+    pur_parser.optopt.add_argument("-x", "--extra_info", required=False, default=None,
+                                   help="File mapping header prefixes to description information.")
+    pur_parser.optopt.add_argument("--stage", default="continue", required=False,
+                                   choices=["continue", "lineages", "classify", "calculate"],
+                                   help="The stage(s) for TreeSAPP to execute [DEFAULT = continue]")
+    # TODO: Remove --trim_align from command-line options in pur_parser.add_seq_params()
     return
 
 
-def add_evaluate_arguments(parser: TreeSAPPArgumentParser) -> None:
+def add_evaluate_arguments(eval_parser: TreeSAPPArgumentParser) -> None:
     """
     Adds command-line arguments that are specific to *treesapp evaluate*
 
     :return: None
     """
-    parser.add_io()
-    parser.add_seq_params()
-    parser.add_refpkg_file_param()
-    parser.add_accession_params()
-    parser.add_compute_miscellany()
-    parser.add_taxa_ranks_param()
+    eval_parser.add_io()
+    eval_parser.add_seq_params()
+    eval_parser.add_refpkg_file_param()
+    eval_parser.add_accession_params()
+    eval_parser.add_compute_miscellany()
+    eval_parser.add_taxa_ranks_param()
 
-    parser.optopt.add_argument("--fresh", default=False, required=False, action="store_true",
-                               help="Recalculate a fresh phylogenetic tree with the target clades removed instead of"
-                                    " removing the leaves corresponding to targets from the reference tree.")
-    parser.optopt.add_argument("--tool", default="treesapp", required=False,
-                               choices=["treesapp", "graftm", "diamond"],
-                               help="Classify using one of the tools: treesapp [DEFAULT], graftm, or diamond.")
-    parser.optopt.add_argument("-l", "--length",
-                               required=False, type=int, default=0,
-                               help="Arbitrarily slice the input sequences to this length. "
-                                    "Useful for testing classification accuracy for fragments.")
-    parser.optopt.add_argument("--stage", default="continue", required=False,
-                               choices=["continue", "lineages", "classify", "calculate"],
-                               help="The stage(s) for TreeSAPP to execute [DEFAULT = continue]")
+    eval_parser.optopt.add_argument("--fresh", default=False, required=False, action="store_true",
+                                    help="Recalculate a fresh phylogenetic tree with the target clades removed instead"
+                                         " of removing the leaves corresponding to targets from the reference tree.")
+    eval_parser.optopt.add_argument("--tool", default="treesapp", required=False,
+                                    choices=["treesapp", "graftm", "diamond"],
+                                    help="Classify using one of the tools: treesapp [DEFAULT], graftm, or diamond.")
+    eval_parser.optopt.add_argument("-l", "--length",
+                                    required=False, type=int, default=0,
+                                    help="Arbitrarily slice the input sequences to this length. "
+                                         "Useful for testing classification accuracy for fragments.")
+    eval_parser.optopt.add_argument("--stage", default="continue", required=False,
+                                    choices=["continue", "lineages", "classify", "calculate"],
+                                    help="The stage(s) for TreeSAPP to execute [DEFAULT = continue]")
     return
 
 
-def add_update_arguments(parser: TreeSAPPArgumentParser) -> None:
+def add_update_arguments(up_parser: TreeSAPPArgumentParser) -> None:
     """
     Adds command-line arguments that are specific to *treesapp update*
 
     :return: None
     """
-    parser.add_output_dir()
-    parser.add_delete()
-    parser.add_seq_params()  # w, m
-    parser.add_cluster_args()  # p
-    parser.add_taxa_args()  # s, f, t
-    parser.add_refpkg_file_param()  # r
-    parser.add_lineage_table_param()
-    parser.add_phylogeny_params()  # b, e
-    parser.add_compute_miscellany()  # n
-    parser.add_basic_classifier_model_params()
-    parser.reqs.add_argument("--treesapp_output", dest="ts_out", required=True,
-                             help="Path to the directory containing TreeSAPP outputs, "
-                                  "including sequences to be used for the update.")
-    parser.optopt.add_argument('-i', '--fastx_input', required=False, dest="input", default="",
-                               help='An input file containing candidate reference sequences in either FASTA format. '
-                                    'Will trigger re-training the reference package if provided.')
-    parser.optopt.add_argument("-l", "--min_lwr", dest="min_lwr", required=False, default=0.0, type=float,
-                               help="The minimum likelihood weight ratio for a sequence to be included in update.")
-    parser.optopt.add_argument("--skip_assign", default=False, required=False, action="store_true",
-                               help="The assigned sequences are from a database and their database lineages "
-                                    "should be used instead of the TreeSAPP-assigned lineages.")
-    parser.optopt.add_argument("--resolve", default=False, required=False, action="store_true",
-                               help="Flag indicating candidate references with better resolved lineages and"
-                                    " comparable sequence lengths can replace old references."
-                                    " Useful when updating with sequences from isolates, SAGs and maybe quality MAGs.")
-    parser.optopt.add_argument("--stage", default="continue", required=False,
-                               choices=["continue", "lineages", "rebuild"],
-                               help="The stage(s) for TreeSAPP to execute [DEFAULT = continue]")
-    parser.miscellany.add_argument("--headless", action="store_true", default=False,
-                                   help="Do not require any user input during runtime.")
+    up_parser.add_output_dir()
+    up_parser.add_delete()
+    up_parser.add_seq_params()  # w, m
+    up_parser.add_cluster_args()  # p
+    up_parser.add_taxa_args()  # s, f, t
+    up_parser.add_refpkg_file_param()  # r
+    up_parser.add_pplace_filter_params()
+    up_parser.add_lineage_table_param()
+    up_parser.add_phylogeny_params()  # b, e
+    up_parser.add_compute_miscellany()  # n
+    up_parser.add_basic_classifier_model_params()
+    up_parser.reqs.add_argument("--treesapp_output", dest="ts_out", required=True,
+                                help="Path to the directory containing TreeSAPP outputs, "
+                                     "including sequences to be used for the update.")
+    up_parser.optopt.add_argument('-i', '--fastx_input', required=False, dest="input", default=[''], nargs='+',
+                                  help='An input file containing candidate reference sequences in either FASTA format. '
+                                       'Will trigger re-training the reference package if provided.')
+    up_parser.optopt.add_argument("--skip_assign", default=False, required=False, action="store_true",
+                                  help="The assigned sequences are from a database and their database lineages "
+                                       "should be used instead of the TreeSAPP-assigned lineages.")
+    up_parser.optopt.add_argument("--resolve", default=False, required=False, action="store_true",
+                                  help="Flag indicating candidate references with better resolved lineages and"
+                                       " comparable sequence lengths can replace old references."
+                                       " Useful when updating with sequences from isolates, SAGs and quality MAGs.")
+    up_parser.optopt.add_argument("--stage", default="continue", required=False,
+                                  choices=["continue", "lineages", "rebuild"],
+                                  help="The stage(s) for TreeSAPP to execute [DEFAULT = continue]")
+    up_parser.miscellany.add_argument("--headless", action="store_true", default=False,
+                                      help="Do not require any user input during runtime.")
     return
 
 
@@ -557,7 +570,7 @@ def check_evaluate_arguments(evaluator_instance: Evaluator, args) -> None:
     for rank in args.taxon_rank:
         evaluator_instance.ranks.append(rank)
 
-    evaluator_instance.ref_pkg.f__json = args.pkg_path
+    evaluator_instance.ref_pkg.f__pkl = args.pkg_path
     evaluator_instance.ref_pkg.slurp()
 
     if args.length:
@@ -579,119 +592,20 @@ def check_evaluate_arguments(evaluator_instance: Evaluator, args) -> None:
     return
 
 
-def check_trainer_arguments(phy_trainer: PhyTrainer, args):
-    phy_trainer.ref_pkg.f__json = args.pkg_path
-    phy_trainer.ref_pkg.slurp()
-    phy_trainer.ref_pkg.validate()
-
-    # Make the directory for storing intermediate outputs
-    if not os.path.isdir(phy_trainer.var_output_dir):
-        os.makedirs(phy_trainer.var_output_dir)
-
-    for rank in args.taxon_rank:
-        phy_trainer.training_ranks[rank] = phy_trainer.ref_pkg.taxa_trie.accepted_ranks_depths[rank]
-
-    # Check whether the parameters for the classifier make sense
-    if args.classifier == "bin":
-        if not args.annot_map:
-            logging.error("An annotation mapping file is required when building a binary classifier.\n")
-            sys.exit(3)
-        else:
-            phy_trainer.annot_map = args.annot_map
-    elif args.classifier == "occ" and args.annot_map:
-        logging.warning("Annotation mapping file is ignored when building a One-Class Classifier (OCC).\n")
-
-    if args.tsne:
-        phy_trainer.tsne_plot = os.path.join(phy_trainer.var_output_dir, "train", "tSNE.png")
-
-    return
-
-
-def check_create_arguments(creator: Creator, args) -> None:
-    # Populate ReferencePackage attributes from command-line arguments
-    if args.fast:
-        creator.ref_pkg.tree_tool = "FastTree"
-    else:
-        creator.ref_pkg.tree_tool = "RAxML-NG"
-    creator.ref_pkg.prefix = args.refpkg_name
-    creator.ref_pkg.pid = args.similarity
-    creator.ref_pkg.molecule = args.molecule
-    creator.ref_pkg.kind = args.kind
-    creator.ref_pkg.sub_model = args.raxml_model
-    creator.ref_pkg.date = dt.now().strftime("%Y-%m-%d")
-    creator.ref_pkg.f__json = creator.final_output_dir + creator.ref_pkg.prefix + creator.ref_pkg.refpkg_suffix
-    # TODO: Create placement trainer output directory and make it an attribute
-    if not args.output:
-        args.output = os.getcwd() + os.sep + creator.ref_pkg.prefix + "_treesapp_refpkg" + os.sep
-
-    if len(creator.ref_pkg.prefix) > 10:
-        logging.error("Name should be <= 10 characters.\n")
-        sys.exit(13)
-
-    # TODO: Check the substitution model for compatibility with RAxML-NG
-
-    if args.cluster:
-        if args.multiple_alignment:
-            logging.error("--cluster and --multiple_alignment are mutually exclusive!\n")
-            sys.exit(13)
-        if not 0.5 <= float(args.similarity) <= 1.0:
-            if 0.5 < float(args.similarity)/100 < 1.0:
-                args.similarity = str(float(args.similarity)/100)
-                logging.warning("--similarity  set to {} for compatibility with VSEARCH.\n".format(args.similarity))
-            else:
-                logging.error("--similarity {} is not between the supported range [0.5-1.0].\n".format(args.similarity))
-                sys.exit(13)
-
-    if args.taxa_lca and not args.cluster:
-        logging.error("Unable to perform LCA for representatives without clustering information: " +
-                      "either with a provided VSEARCH file or by clustering within the pipeline.\n")
-        sys.exit(13)
-
-    if args.guarantee and not args.cluster:
-        logging.error("--guarantee used but without clustering there is no reason for it.\n" +
-                      "Include all sequences in " + args.guarantee +
-                      " in " + creator.input_sequences + " and re-run without --guarantee\n")
-        sys.exit(13)
-
-    if args.profile:
-        if not os.path.isfile(args.profile):
-            logging.error("Unable to find HMM profile at '" + args.profile + "'.\n")
-            sys.exit(3)
-        creator.hmm_profile = args.profile
-
-    # Names of files and directories to be created
-    creator.phy_dir = os.path.abspath(creator.var_output_dir) + os.sep + "phylogeny_files" + os.sep
-    creator.training_dir = os.path.abspath(creator.var_output_dir) + os.sep + "placement_trainer" + os.sep
-    creator.hmm_purified_seqs = creator.var_output_dir + creator.ref_pkg.prefix + "_hmm_purified.fasta"
-    creator.filtered_fasta = creator.var_output_dir + creator.sample_prefix + "_filtered.fa"
-    creator.cluster_input = creator.var_output_dir + creator.sample_prefix + "_cluster_input.fasta"
-    creator.clusters_prefix = creator.var_output_dir + creator.sample_prefix + "_cluster" + str(creator.ref_pkg.pid)
-    creator.unaln_ref_fasta = creator.var_output_dir + creator.ref_pkg.prefix + "_ref.fa"
-    creator.phylip_file = creator.var_output_dir + creator.ref_pkg.prefix + ".phy"
-
-    # Ensure the phylogenetic tree output directory from a previous run isn't going to be over-written
-    if not os.path.exists(creator.phy_dir):
-        os.mkdir(creator.phy_dir)
-    else:
-        logging.error(creator.phy_dir + " already exists from a previous run! " +
-                      "Please delete or rename it and try again.\n")
-        sys.exit(13)
-
-    if not os.path.isdir(creator.training_dir):
-        os.mkdir(creator.training_dir)
-
-    return
-
-
 def check_updater_arguments(updater: Updater, args):
-    updater.ref_pkg.f__json = args.pkg_path
+    updater.ref_pkg.f__pkl = args.pkg_path
     updater.ref_pkg.slurp()
     updater.updated_refpkg_path = os.path.join(updater.output_dir, "final_outputs",
-                                               os.path.basename(updater.ref_pkg.f__json))
+                                               os.path.basename(updater.ref_pkg.f__pkl))
     updater.ref_pkg.disband(os.path.join(updater.output_dir, "intermediates"))
     updater.seq_names_to_taxa = args.seq_names_to_taxa
     # updater.rank_depth_map = {'k': 1, 'p': 2, 'c': 3, 'o': 4, 'f': 5, 'g': 6, 's': 7}
 
+    if updater.input_sequences:
+        updater.find_sequence_molecule_type()
+    else:
+        updater.molecule_type = updater.ref_pkg.molecule
+
     if args.similarity == 1.0:
         updater.prop_sim = updater.ref_pkg.pid
     else:
@@ -722,7 +636,7 @@ def check_updater_arguments(updater: Updater, args):
     updater.old_ref_fasta = updater.var_output_dir + "original_refs.fasta"
     updater.combined_fasta = updater.var_output_dir + "all_refs.fasta"
     updater.lineage_map_file = updater.var_output_dir + "accession_id_lineage_map.tsv"
-    updater.assignment_table = updater.final_output_dir + "marker_contig_map.tsv"
+    updater.assignment_table = updater.final_output_dir + updater.classification_tbl_name
     updater.cluster_input = updater.var_output_dir + updater.sample_prefix + "_cluster_input.fasta"
     updater.clusters_prefix = updater.var_output_dir + updater.sample_prefix + "_cluster" + str(updater.prop_sim)
     classified_seqs = glob(updater.final_output_dir + "*_classified.faa")
