@@ -3,13 +3,13 @@ __author__ = 'Connor Morgan-Lang'
 import os
 import re
 import sys
-import subprocess
 import logging
 import shutil
 from glob import glob
 from csv import Sniffer
 
 from pygtrie import StringTrie
+import multiprocessing
 
 from treesapp.external_command_interface import launch_write_command
 
@@ -132,8 +132,12 @@ def available_cpu_count():
     user/real as output by time(1) when called with an optimally scaling
     userspace-only program"""
 
-    # cpuset
-    # cpuset may restrict the number of *available* processors
+    try:
+        res = multiprocessing.cpu_count()
+        return res
+    except (ImportError, NotImplementedError):
+        pass
+
     try:
         m = re.search(r'(?m)^Cpus_allowed:\s*(.*)$',
                       open('/proc/self/status').read())
@@ -149,76 +153,8 @@ def available_cpu_count():
     except (ImportError, NotImplementedError):
         pass
 
-    # POSIX
-    try:
-        res = int(os.sysconf('SC_NPROCESSORS_ONLN'))
-
-        if res > 0:
-            return res
-    except (AttributeError, ValueError):
-        pass
-
-    # Windows
-    try:
-        res = int(os.environ['NUMBER_OF_PROCESSORS'])
-
-        if res > 0:
-            return res
-    except (KeyError, ValueError):
-        pass
-
-    # BSD
-    try:
-        sysctl = subprocess.Popen(['sysctl', '-n', 'hw.ncpu'],
-                                  stdout=subprocess.PIPE)
-        scStdout = sysctl.communicate()[0]
-        res = int(scStdout)
-
-        if res > 0:
-            return res
-    except (OSError, ValueError):
-        pass
-
-    # Linux
-    try:
-        res = open('/proc/cpuinfo').read().count('processor\t:')
-
-        if res > 0:
-            return res
-    except IOError:
-        pass
-
-    # Solaris
-    try:
-        pseudoDevices = os.listdir('/devices/pseudo/')
-        res = 0
-        for pd in pseudoDevices:
-            if re.match(r'^cpuid@[0-9]+$', pd):
-                res += 1
-
-        if res > 0:
-            return res
-    except OSError:
-        pass
-
-    # Other UNIXes (heuristic)
-    try:
-        try:
-            dmesg = open('/var/run/dmesg.boot').read()
-        except IOError:
-            dmesgProcess = subprocess.Popen(['dmesg'], stdout=subprocess.PIPE)
-            dmesg = dmesgProcess.communicate()[0]
-
-        res = 0
-        while '\ncpu' + str(res) + ':' in dmesg:
-            res += 1
-
-        if res > 0:
-            return res
-    except OSError:
-        pass
-
     logging.error('Can not determine number of CPUs on this system')
+    return
 
 
 def executable_dependency_versions(exe_dict: dict) -> str:
@@ -308,38 +244,6 @@ def mean(num_list: list):
     return float(sum(num_list) / len(num_list))
 
 
-def convert_outer_to_inner_nodes(clusters: dict, internal_node_map: dict):
-    """
-    Find the lowest common ancestor (internal node) for all leaves in the range.
-    This is only necessary if the original nodes parsed from the colours_style.txt file were leaves.
-
-    :param clusters: A dictionary mapping start and end leaves of a clade for a single marker's colours_style.txt layer
-    :param internal_node_map: A dictionary mapping each internal node to a list of all of its descendent leaves
-    :return: A dictionary of annotation strings mapped to a list of internal nodes
-    """
-    leaf_annotation_map = dict()
-    for annotation in clusters:
-        leaf_annotation_map[annotation] = list()
-        for leaf_nodes in clusters[annotation]:
-            start, end = leaf_nodes
-            try:
-                if int(start) == int(end) and int(start) in internal_node_map:
-                    leaf_annotation_map[annotation].append(int(start))
-            except ValueError:
-                # Find the minimum set that includes both start and end
-                warm_front = dict()
-                # Add all the potential internal nodes
-                for inode in internal_node_map:
-                    clade = internal_node_map[inode]
-                    if start in clade:
-                        warm_front[inode] = clade
-                for inode in sorted(warm_front, key=lambda x: len(warm_front[x])):
-                    if end in warm_front[inode]:
-                        leaf_annotation_map[annotation].append(inode)
-                        break
-    return leaf_annotation_map
-
-
 def reformat_fasta_to_phy(fasta_dict: dict) -> dict:
     """
     The fasta_dict input is a dictionary of sequence names (seq_name) keys indexing their respective sequences.
@@ -534,6 +438,8 @@ def concatenate_files(input_files: list, output_path: str):
         logging.error("Unable to open output file '{}' for writing.\n".format(output_path))
         sys.exit(17)
 
+    if isinstance(input_files, str):
+        raise AssertionError("File list is str, when list was expected.")
     for input_f in input_files:
         with open(input_f, 'rb') as fh:
             shutil.copyfileobj(fh, output_handler)
@@ -557,9 +463,8 @@ def complement_nucs(nuc_str: str):
                 comp_str += 'N'
 
     if replacements:
-        logging.warning(str(len(replacements)) +
-                        " ambiguity character(s) (" + ', '.join(sorted(set(replacements))) +
-                        ") replaced by 'N' while complementing\n")
+        logging.warning("{} ambiguity character(s) ({}) replaced by 'N' while complementing\n"
+                        "".format(len(replacements), ', '.join(sorted(set(replacements)))))
     return comp_str
 
 
@@ -567,7 +472,7 @@ def reverse_complement(nuc_sequence: str):
     return complement_nucs(nuc_sequence[::-1])
 
 
-def find_msa_type(msa_files):
+def find_msa_type(msa_files: dict):
     file_types = set()
     for mc in msa_files:
         sample_msa_file = msa_files[mc][0]
@@ -592,22 +497,6 @@ def find_msa_type(msa_files):
         return file_types.pop()
 
 
-def match_target_marker(refpkg_name: str, headers: list) -> list:
-    """
-    Returns the list of sequences with TreeSAPP classification tags matching the refpkg_name
-
-    :param refpkg_name: The refpkg name (e.g. McrA) not code (e.g. M0701) for desired classified sequences
-    :param headers: List of classified sequences
-    :return: List of headers that match the refpkg_name
-    """
-    matches = list()
-    classified_target = re.compile(r'\|{0}\|\d+_\d+$'.format(re.escape(refpkg_name)))
-    for seq_name in headers:
-        if classified_target.search(seq_name):
-            matches.append(seq_name)
-    return matches
-
-
 def get_file_lines(file_path: str, re_pattern=None, max_matches=0) -> list:
     """
     Opens a file and returns it's contents as a list using readlines().
